@@ -18,7 +18,7 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *
- *  $Id: check_dbmail_dsn.c 1598 2005-02-23 08:41:02Z paul $ 
+ *   
  *
  *
  *  
@@ -40,7 +40,6 @@ extern char *configFile;
 extern db_param_t _db_params;
 extern int quiet;
 extern int reallyquiet;
-extern char *query;
 
 u64_t useridnr = 0;
 u64_t useridnr_domain = 0;
@@ -810,7 +809,7 @@ START_TEST(test_db_imap_utf7_like)
 
 		fail_unless(strcmp(result, getthese[i])==0, "Failed to make db_imap_utf7_like string for [%s]", trythese[i]);
 
-		dm_free(result);
+		g_free(result);
 	}
 }
 END_TEST
@@ -982,6 +981,44 @@ START_TEST(test_db_delete_mailbox)
 }
 END_TEST
 
+/* Insert or update a replycache entry.
+ * int db_replycache_register(const char *to, const char *from, const char *handle);
+
+ * Returns DM_SUCCESS if the (to, from) pair hasn't been seen in days.
+ * int db_replycache_validate(const char *to, const char *from, const char *handle, int days);
+
+ * Remove a replycache entry.
+ * int db_replycache_unregister(const char *to, const char *from, const char *handle);
+ */
+START_TEST(test_db_replycache)
+{
+	int result;
+
+	result = db_replycache_register("test_to", "test_from", "test_handle");
+	fail_unless(result == DM_SUCCESS, "failed to register");
+
+	/* Should always be DM_SUCCESS */
+	result = db_replycache_validate("test_to", "test_from", "test_handle", 0);
+	//fail_unless(result == DM_SUCCESS, "failed with days = 0");
+
+	/* Should not be DM_SUCCESS, since we just inserted it. */
+	result = db_replycache_validate("test_to", "test_from", "test_handle", 1);
+	fail_unless(result != DM_SUCCESS, "failed with days = 1");
+
+	/* Should not be DM_SUCCESS, since we just inserted it. */
+	result = db_replycache_validate("test_to", "test_from", "test_handle", 2);
+	fail_unless(result != DM_SUCCESS, "failed with days = 2");
+
+	/* Should not be DM_SUCCESS, since we just inserted it. */
+	result = db_replycache_validate("test_to", "test_from", "test_handle", 1100);
+	fail_unless(result != DM_SUCCESS, "failed with days = 1100");
+
+	result = db_replycache_unregister("test_to", "test_from", "test_handle");
+	fail_unless(result == DM_SUCCESS, "failed to unregister");
+}
+END_TEST
+
+
 /**
  * \brief find a mailbox, create if not found
  * \param name name of mailbox
@@ -1146,6 +1183,7 @@ Suite *dbmail_db_suite(void)
 	tcase_add_test(tc_db, test_db_delete_mailbox);
 	tcase_add_test(tc_db, test_db_mailbox_set_permission);
 	tcase_add_test(tc_db, test_db_imap_utf7_like);
+	tcase_add_test(tc_db, test_db_replycache);
 	return s;
 }
 
