@@ -17,7 +17,7 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *
- *  $Id: check_dbmail_imapd.c 2371 2006-11-15 15:55:46Z paul $ 
+ *   
  *
  *
  *  
@@ -124,13 +124,14 @@ END_TEST
 
 
 #define A(x,y) s=dbmail_imap_astring_as_string(x); \
-	fail_unless(strcmp(y,s)==0,"dbmail_imap_astring_as_string failed"); \
+	fail_unless(strcmp(y,s)==0,"dbmail_imap_astring_as_string failed [%s] != [%s]", s, y); \
 	g_free(s)
 START_TEST(test_dbmail_imap_astring_as_string)
 {
 	char *s;
 	A("test","\"test\"");
 	A("\"test\"","\"test\"");
+	A("\"test\" \"test\"","{13}\r\n\"test\" \"test\"");
 	A("\"test","{5}\r\n\"test");
 	A("test","{5}\r\ntest");
 	A("test\"","{5}\r\ntest\"");
@@ -203,7 +204,6 @@ END_TEST
 
 
 //dbmail_imap_session_handle_auth(self, const char *username, const char *password);
-#define MAX_LINESIZE 1024
 START_TEST(test_imap_session_prompt)
 {
 	int result;
@@ -302,10 +302,10 @@ START_TEST(test_dbmail_imap_session_fetch_get_items)
 	result = dbmail_imap_session_mailbox_open(s,mailbox);
 	fail_unless(result==0, "mailbox_open failed");
 
-	dbmail_mailbox_set_uid(s->mailbox,TRUE);
-	s->fetch_ids = dbmail_mailbox_get_set(s->mailbox,"1:*",TRUE);
+	dbmail_mailbox_set_uid(s->mailbox, TRUE);
+	s->ids = dbmail_mailbox_get_set(s->mailbox, "1:*", TRUE);
 	
-	fail_unless(s->fetch_ids!=NULL, "get_set failed");
+	fail_unless(s->ids!=NULL, "get_set failed");
 
 	result = dbmail_imap_session_fetch_get_items(s);
 	fail_unless(result==0, "fetch_get_items failed");
@@ -327,7 +327,8 @@ START_TEST(test_imap_bodyfetch)
 	fail_unless(0 == dbmail_imap_session_bodyfetch_get_last_octetcnt(s), "octetcnt init value incorrect");
 	fail_unless(0 == dbmail_imap_session_bodyfetch_get_last_argstart(s), "argstart init value incorrect");
 	
-	dbmail_imap_session_bodyfetch_set_argstart(s,23);
+	s->args_idx = 23;
+	dbmail_imap_session_bodyfetch_set_argstart(s);
 	result = dbmail_imap_session_bodyfetch_get_last_argstart(s);
 	fail_unless(result==23, "argstart incorrect");
 	
@@ -370,7 +371,6 @@ START_TEST(test_imap_get_structure)
 	message = dbmail_message_new();
 	message = dbmail_message_init_with_string(message, g_string_new(simple));
 	result = imap_get_structure(GMIME_MESSAGE(message->content), 1);
-	printf("[%s]\n", result);
 	dbmail_message_free(message);
 	g_free(result);
 
@@ -381,7 +381,6 @@ START_TEST(test_imap_get_structure)
 	strncpy(expect,"((\"text\" \"html\" NIL NIL NIL \"7BIT\" 16 1 NIL (\"inline\" NIL) NIL NIL)"
 			"(\"text\" \"plain\" (\"charset\" \"us-ascii\" \"name\" \"testfile\") NIL NIL \"base64\" 432 7 NIL NIL NIL NIL)"
 			" \"mixed\" (\"boundary\" \"boundary\") NIL NIL NIL)",1024);
-//	printf("\n[%s]\n[%s]\n", expect, result);
 	fail_unless(strncasecmp(result,expect,1024)==0, "imap_get_structure failed");
 	g_free(result);
 	dbmail_message_free(message);
@@ -437,9 +436,8 @@ START_TEST(test_internet_address_parse_string)
 
 		fail_unless(strcmp(result,expect)==0, "internet_address_parse_string failed to generate correct undisclosed-recipients plist, expected [%s] got [%s]", expect, result);
 
-		g_list_foreach(list,(GFunc)g_free,NULL);
-		g_list_free(list);
-		dm_free(result);
+		g_list_destroy(list);
+		g_free(result);
 
 	}
 
@@ -462,7 +460,6 @@ START_TEST(test_internet_address_parse_string)
 		input = testlist[i][0];
 		expect = testlist[i][1];
 
-		// printf("testing parse of [%s]\n looking for [%s]\n", input, expect);
 		InternetAddressList *alist;
 		GList *list = NULL;
 		char *result;
@@ -470,18 +467,17 @@ START_TEST(test_internet_address_parse_string)
 		char *t;
 		t = imap_cleanup_address(input);
 		alist = internet_address_parse_string(t);
-		g_free(t);
 		list = dbmail_imap_append_alist_as_plist(list, (const InternetAddressList *)alist);
 		result = dbmail_imap_plist_as_string(list);
 		res = strcmp(result, expect);
-		if (res == 0)
-			g_free(result); // Only free if we're not going to report an error.
+
+		fail_unless(res == 0, "dbmail_imap_append_alist_as_plist failed, expected:\n[%s]\ngot:\n[%s]\n", expect, result);
+
 		internet_address_list_destroy(alist);
 		alist = NULL;
-		g_list_foreach(list, (void *)g_free, NULL);
-		g_list_free(list);
-
-		fail_unless(res == 0, "dbmail_imap_append_alist_as_plist failed, expected [%s] got [%s]", expect, result);
+		g_list_destroy(list);
+		g_free(result);
+		g_free(t);
 	}
 }
 END_TEST
@@ -490,12 +486,15 @@ START_TEST(test_imap_get_envelope)
 {
 	struct DbmailMessage *message;
 	char *result, *expect;
+	GString *s;
 	
 	expect = g_new0(char, 1024);
 	
 	/* text/plain */
 	message = dbmail_message_new();
-	message = dbmail_message_init_with_string(message, g_string_new(rfc822));
+	s = g_string_new(rfc822);
+	message = dbmail_message_init_with_string(message, s);
+	g_string_free(s,TRUE);
 	result = imap_get_envelope(GMIME_MESSAGE(message->content));
 	strncpy(expect,"(\"Thu, 01 Jan 1970 00:00:00 +0000\" \"dbmail test message\" ((NIL NIL \"somewher\" \"foo.org\")) ((NIL NIL \"somewher\" \"foo.org\")) ((NIL NIL \"somewher\" \"foo.org\")) ((NIL NIL \"testuser\" \"foo.org\")) NIL NIL NIL NIL)",1024);
 	fail_unless(strncasecmp(result,expect,1024)==0, "imap_get_envelope failed");
@@ -503,19 +502,24 @@ START_TEST(test_imap_get_envelope)
 	dbmail_message_free(message);
 	g_free(result);
 	result = NULL;
-	g_free(expect);
-	expect = NULL;
 
 	/* bare bones message */
 	message = dbmail_message_new();
-	message = dbmail_message_init_with_string(message, g_string_new(simple));
+	s = g_string_new(simple);
+	message = dbmail_message_init_with_string(message, s);
+	g_string_free(s,TRUE);
 	result = imap_get_envelope(GMIME_MESSAGE(message->content));
 
-	printf("[%s]\n", result);
+	strncpy(expect,"(\"Thu, 01 Jan 1970 00:00:00 +0000\" \"dbmail test message\" NIL NIL NIL NIL NIL NIL NIL NIL)", 1024);
+	fail_unless(strncasecmp(result,expect,1024)==0, "imap_get_envelope failed");
+
 	dbmail_message_free(message);
 	g_free(result);
 	result = NULL;
 
+	g_free(expect);
+	expect = NULL;
+
 }
 END_TEST
 
@@ -575,6 +579,7 @@ END_TEST
 
 			
 #define F(a,b) fail_unless(strcmp(c = imap_cleanup_address(a), b)==0, "\n[" a "] should have yielded \n[" b "] but got \n[%s]", c)
+#define Fnull(a,b) fail_unless(strcmp(c = imap_cleanup_address(a), b)==0, "\n[] should have yielded \n[" b "] but got \n[%s]", c)
 	
 START_TEST(test_imap_cleanup_address)
 {
@@ -584,11 +589,16 @@ START_TEST(test_imap_cleanup_address)
 	F("=?iso-8859-1?Q?\"B=BA_V._F._Z=EAzere\"?=<nobody@nowhere.org>","\"=?iso-8859-1?Q?B=BA_V._F._Z=EAzere?=\" <nobody@nowhere.org>");
 	F("=?iso-8859-1?Q?B=BA_V._F._Z=EAzere?=<nobody@nowhere.org>","\"=?iso-8859-1?Q?B=BA_V._F._Z=EAzere?=\" <nobody@nowhere.org>");
 	F("\"=?iso-8859-1?Q?B=BA_V._F._Z=EAzere?=\" <nobody@nowhere.org>","\"=?iso-8859-1?Q?B=BA_V._F._Z=EAzere?=\" <nobody@nowhere.org>");
+	F("", "");
+	Fnull(NULL, "");
 	F("Some One <some@foo.org>", "Some One <some@foo.org>");
 	F(" <some@foo.org>", "<some@foo.org>");
 	F("=?ISO-8859-2?Q? \"Verlag=20Dash=F6fer=20-=20DU.cz?= =?ISO-8859-2?Q?\" ?= <e-noviny@smtp.dashofer.cz>",
 	"\"=?ISO-8859-2?Q?Verlag=20Dash=F6fer=20-=20DU.cz?= =?ISO-8859-2?Q?"
 		/* Stringify here to kill the '??=' trigraph. */ "?=\" <e-noviny@smtp.dashofer.cz>");
+	F("=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\n"
+	"	<fordfrog@gentoo.org>\n", "\"=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\" <fordfrog@gentoo.org>");
+
 
 }
 END_TEST
@@ -597,9 +607,13 @@ START_TEST(test_imap_get_envelope_latin)
 {
 	char *t;
 	char *expect = g_new0(char,1024);
-	struct DbmailMessage *m = dbmail_message_new();
-	GString *s = g_string_new(encoded_message_latin);
+	struct DbmailMessage *m;
+	GString *s;
+	
 
+	/*  */
+	m = dbmail_message_new();
+	s = g_string_new(encoded_message_latin_1);
 	m = dbmail_message_init_with_string(m, s);
 	g_string_free(s,TRUE);
 	
@@ -611,6 +625,23 @@ START_TEST(test_imap_get_envelope_latin)
 
 	g_free(t);
 	dbmail_message_free(m);
+
+	/*  */
+	m = dbmail_message_new();
+	s = g_string_new(encoded_message_latin_2);
+	m = dbmail_message_init_with_string(m, s);
+	g_string_free(s,TRUE);
+	
+	strncpy(expect,"(\"Thu, 01 Jan 1970 00:00:00 +0000\" \"=?ISO-8859-2?Q?Re=3A_=5Bgentoo-dev=5D_New_developer=3A__?= =?ISO-8859-2?Q?Miroslav_=A9ulc_=28fordfrog=29?=\" ((\"Miroslav  =?iso-8859-2?b?qXVsYw==?=  (fordfrog)\" NIL \"fordfrog\" \"gentoo.org\")) ((\"Miroslav  =?iso-8859-2?b?qXVsYw==?=  (fordfrog)\" NIL \"fordfrog\" \"gentoo.org\")) ((\"Miroslav  =?iso-8859-2?b?qXVsYw==?=  (fordfrog)\" NIL \"fordfrog\" \"gentoo.org\")) ((NIL NIL \"gentoo-dev\" \"lists.gentoo.org\")) NIL NIL NIL NIL)",1024);
+
+	t = imap_get_envelope(GMIME_MESSAGE(m->content));
+	fail_unless(strcmp(t,expect)==0,"imap_get_envelope failed\n%s\n%s\n ", expect, t);
+	
+	g_free(t);
+	g_free(expect);
+	dbmail_message_free(m);
+
+
 }
 END_TEST
 
@@ -819,12 +850,10 @@ unsigned int get_count_on(unsigned int * set, unsigned int setlen) {
 	
 static int wrap_base_subject(const char *in, const char *expect) 
 {
-	char *s = g_strdup(in);
-	char *out = s;
+	char *out = dm_base_subject(in);
 	int res;
-	dm_base_subject(out);
 	res = strcmp(out, expect);
-	g_free(s);
+	g_free(out);
 	return res;
 }	
 
