@@ -19,8 +19,6 @@
 */
 
 /*
- * $Id: db.h 2309 2006-10-20 19:14:52Z aaron $
- *
  * dbase driver header file
  * Functions for database communication 
  *
@@ -616,13 +614,13 @@ int db_log_ip(const char *ip);
 /**
 * \brief clean up ip log
 * \param lasttokeep all ip log entries before this timestamp will
-*                     deleted
+*                     be deleted
 * \return 
 *       - -1 on database failure
 *       - 0 on success
 */
-int db_cleanup_iplog(const char *lasttokeep, u64_t *affected_rows);
-int db_count_iplog(const char *lasttokeep, u64_t *affected_rows);
+int db_cleanup_iplog(timestring_t lasttokeep, u64_t *affected_rows);
+int db_count_iplog(timestring_t lasttokeep, u64_t *affected_rows);
 
 /**
  * \brief cleanup database tables
@@ -1183,17 +1181,6 @@ int db_unsubscribe(u64_t mailbox_idnr, u64_t user_idnr);
 int db_get_msgflag(const char *flag_name,
 		   u64_t msg_idnr, u64_t mailbox_idnr);
 
-/**
- * \brief get all flags for a message
- * \param msg_idnr
- * \param mailbox_idnr
- * \param flags An array of IMAP_NFLAGS elements. 
- * \return 
- * 		- -1 on failure
- * 		- 0 on success
- */
-int db_get_msgflag_all(u64_t msg_idnr, u64_t mailbox_idnr, int *flags);
-
 /**
  * \brief set flags for a message
  * \param msg_idnr
@@ -1213,89 +1200,6 @@ int db_get_msgflag_all(u64_t msg_idnr, u64_t mailbox_idnr, int *flags);
  */
 int db_set_msgflag(u64_t msg_idnr, u64_t mailbox_idnr, int *flags,
 		   int action_type);
-/**
- * \brief set flags for a range of messages in a mailbox
- * \param msg_idnr_low beginning of range
- * \param msg_idnr_high end of range
- * \param mailbox_idnr
- * \param flags Array of IMAP_NFLAGS elements. See
- *               imapcommand.c for details.
- * \param action_type 
- *        - IMAPFA_REPLACE new set will be exactly as flags,
- *          with '1' for set, and '0' for not set.
- *        - IMAPFA_ADD set all flags which have a '1' as value
- *          in flags
- *        - IMAPFA_REMOVE clear all flags that have '1' as value
- *          in flags
- * \return 
- * 		- -1 on failure
- * 		-  0 on success
- */
-int db_set_msgflag_range(u64_t msg_idnr_low,
-			 u64_t msg_idnr_high, u64_t mailbox_idnr,
-			 int *flags, int action_type);
-
-int db_set_msgflag_recent(u64_t msg_idnr, u64_t mailbox_idnr);
-
-int db_set_msgflag_recent_range(u64_t msg_idnr_lo, u64_t msg_idnr_hi, u64_t mailbox_idnr);
-
-
-/**
- * \brief retrieve internal message date
- * \param mailbox_idnr
- * \param msg_idnr
- * \param date string of size IMAP_INTERNALDATE_LEN which will
- *        hold the date after call.
- * \return 
- *     - -1 on failure
- *     -  0 on success
- */
-int db_get_msgdate(u64_t mailbox_idnr, u64_t msg_idnr, char *date);
-/**
- * \brief set the RFCSIZE field of a message
- * \param rfcsize new rfc size
- * \param msg_idnr
- * \param mailbox_idnr
- * \return
- * 		- -1 on failure
- * 		- 0 on success
- */
-int db_set_rfcsize(u64_t rfcsize, u64_t msg_idnr, u64_t mailbox_idnr);
-/**
- * \brief get the RFCSIZE field of a message
- * \param msg_idnr
- * \param mailbox_idnr
- * \param rfc_size will hold RFCSIZE after return. Must be a valid pointer
- * on call.
- * \return
- * 		- -1 on failure
- * 		- 1 on success
- */
-int db_get_rfcsize(u64_t msg_idnr, u64_t mailbox_idnr, u64_t * rfc_size);
-
-/**
- * \brief builds a list containing the fields of
- * the main header.
- * \param msg_idnr
- * \param hdrlist will hold the list when finished
- * \return
- *    - -3 parse error
- *    - -2 memory error
- *    - -1 database error
- *    - 0 success
- * \attention hdrlist should be empty on call.
- */
-int db_get_main_header(u64_t msg_idnr, struct dm_list *hdrlist, const char *headername);
-/**
- * \brief check if a message belongs to a mailbox
- * \param mailbox_idnr
- * \param message_idnr
- * \return 
- *    - -1 on failure
- *    -  0 if message does not belong to mailbox
- *    - 1 if message belongs to mailbox
- */
-int db_mailbox_msg_match(u64_t mailbox_idnr, u64_t message_idnr);
 
 /**
  * \brief check if a user has a certain right to a mailbox
@@ -1382,10 +1286,20 @@ int db_user_log_login(u64_t user_idnr);
 int db_change_mailboxsize(u64_t user_idnr, u64_t new_size);
 
 /* auto-reply cache */
-int db_replycache_register(const char *to, const char *from,
-		const char *handle);
-int db_replycache_validate(const char *to, const char *from,
-		const char *handle, int days);
+int db_replycache_register(const char *to, const char *from, const char *handle);
+int db_replycache_validate(const char *to, const char *from, const char *handle, int days);
+int db_replycache_unregister(const char *to, const char *from, const char *handle);
+
+/**
+* \brief clean up replycache
+* \param lasttokeep all replycache entries before this timestamp will
+*                     be deleted
+* \return 
+*       - -1 on database failure
+*       - 0 on success
+*/
+int db_cleanup_replycache(timestring_t lasttokeep, u64_t *affected_rows);
+int db_count_replycache(timestring_t lasttokeep, u64_t *affected_rows);
 
 /* get driver specific SQL snippets */
 const char * db_get_sql(sql_fragment_t frag);
