@@ -17,7 +17,7 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* $Id: user.c 1665 2005-03-12 17:42:59Z paul $
+/* 
  * This is the dbmail-user program
  * It makes adding users easier */
 
@@ -41,11 +41,11 @@ static const char ValidChars[] =
     "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
     "_.!@#$%^&*()-+=~[]{}<>:;\\/";
 
-int yes_to_all = 0;
-int no_to_all = 0;
-int verbose = 0;
-int quiet = 0; 		/* Don't be helpful. */
-int reallyquiet = 0;	/* Don't print errors. */
+extern int yes_to_all;
+extern int no_to_all;
+extern int verbose;
+extern int quiet; 		/* Don't be helpful. */
+extern int reallyquiet;	/* Don't print errors. */
 
 int do_add(const char * const user,
            const char * const password, const char * const enctype,
@@ -55,53 +55,44 @@ int do_add(const char * const user,
 {
 	u64_t useridnr;
 	u64_t mailbox_idnr;
-	int add_user_result, result;
+	int result;
 
-	if (!is_valid(user)) {
-		qerrorf("Error: invalid characters in username [%s]\n",
-		     user);
-		return -1;
+	if (no_to_all) {
+		qprintf("Pretending to add user %s with password type %s, %llu bytes mailbox limit and clientid %llu\n",
+			user, enctype, maxmail, clientid);
+		return 1;
 	}
 
 	TRACE(TRACE_DEBUG, "Adding user %s with password type %s,%llu "
 		"bytes mailbox limit and clientid %llu... ", 
 		user, enctype, maxmail, clientid);
 
-	switch (auth_user_exists(user, &useridnr))
-	{
-	case -1:
-		/* Database failure */
-		qerrorf("Failed\n\nCheck logs for details\n\n");
+	if ((result = auth_user_exists(user, &useridnr))) {
+		qerrorf("Failed: check logs for details\n");
+		return result;
+	}
+
+	if (auth_adduser(user, password, enctype, clientid, maxmail, &useridnr) < 0) {
+		qerrorf("Failed: unable to create user\n");
 		return -1;
-	default:
-		if (useridnr != 0) {
-			qprintf("Failed: user exists [%llu]\n",
-				     useridnr);
-			return -1;
-		} else {
-			/* If useridnr is 0, create the user */
-			add_user_result = auth_adduser(user, password, enctype,
-				clientid, maxmail, &useridnr);
-		}
-		break;
 	}
 
 	TRACE(TRACE_DEBUG, "Ok, user added id [%llu]\n", useridnr);
 
 	/* Add an INBOX for the user. */
-	qprintf("Adding INBOX for new user\n");
-	switch(db_createmailbox("INBOX", useridnr, &mailbox_idnr)) {
-	case -1:
-		qprintf("Failed... User is added but we failed to add "
-			     "the mailbox INBOX for this user\n");
-		result = -1;
-		break;
-	case 0:
-	default:
-		TRACE(TRACE_DEBUG, "Ok. added\n");
-		result = 0;
-		break;
-	} 
+	qprintf("Adding INBOX for new user... ");
+
+	if (db_createmailbox("INBOX", useridnr, &mailbox_idnr) < 0) {
+		qprintf("failed... removing user... ");
+		if (auth_delete_user(user))
+			qprintf("failed also.\n");
+		else
+			qprintf("done.\n");
+		return -1;
+	}
+	qprintf("ok.\n");
+
+	TRACE(TRACE_DEBUG, "Ok. INBOX created for user.\n");
 
 	if(do_aliases(useridnr, alias_add, alias_del) < 0)
 		result = -1;
@@ -116,16 +107,17 @@ int do_username(const u64_t useridnr, const char * const newuser)
 {
 	int result = 0;
 
-	if (newuser && is_valid(newuser)) {
-		if (auth_change_username(useridnr, newuser) != 0) {
-			qerrorf("Error: could not change username.\n");
-			result = -1;
-		}
-	} else {
-		qerrorf("Error: new username contains invalid characters.\n");
-		result = -1;
+	assert(newuser);
+
+	if (no_to_all) {
+		qprintf("Pretending to change username of user id number [%llu] to [%s]\n",
+			useridnr, newuser);
+		return 1;
 	}
 
+	if ((result = auth_change_username(useridnr, newuser)))
+		qerrorf("Error: could not change username.\n");
+
 	return result;
 }
 
@@ -134,12 +126,16 @@ int do_password(const u64_t useridnr,
                 const char * const password, const char * const enctype)
 {
 	int result = 0;
-
-	result = auth_change_password(useridnr, password, enctype);
-	if (result != 0) {
-		qerrorf("Error: could not change password.\n");
+	
+	if (no_to_all) {
+		qprintf("Pretending to change password for user id number [%llu]\n",
+			useridnr);
+		return 1;
 	}
 
+	if ((result = auth_change_password(useridnr, password, enctype)))
+		qerrorf("Error: could not change password.\n");
+
 	return result;
 }
 
@@ -211,7 +207,7 @@ int mkpassword(const char * const user, const char * const passwd,
 			md5str = dm_md5((unsigned char *)passwd);
 			null_strncpy(pw, md5str, 49);
 			*enctype = "md5sum";
-			dm_free(md5str);
+			g_free(md5str);
 			break;
 		case MD5_DIGEST_RAW:
 			null_strncpy(pw, passwd, 49);
@@ -222,7 +218,7 @@ int mkpassword(const char * const user, const char * const passwd,
 			md5str = dm_md5_base64((unsigned char *)passwd);
 			null_strncpy(pw, md5str, 49);
 			*enctype = "md5base64";
-			dm_free(md5str);
+			g_free(md5str);
 			}
 			break;
 		case MD5_BASE64_RAW:
@@ -263,7 +259,7 @@ int mkpassword(const char * const user, const char * const passwd,
 	}
 
 	/* Pass this out of the function. */
-	*password = dm_strdup(pw);
+	*password = g_strdup(pw);
 
 	return result;
 }
@@ -273,11 +269,15 @@ int do_clientid(u64_t useridnr, u64_t clientid)
 {	
 	int result = 0;
 
-	if (auth_change_clientid(useridnr, clientid) != 0) {
-		qprintf("\nWarning: could not change client id ");
-		result = -1;
+	if (no_to_all) {
+		qprintf("Pretending to change client for user id number [%llu] to client id number [%llu]\n",
+			useridnr, clientid);
+		return 1;
 	}
 
+	if ((result = auth_change_clientid(useridnr, clientid)))
+		qerrorf("Warning: could not change client id\n");
+
 	return result;
 }
 
@@ -286,11 +286,15 @@ int do_maxmail(u64_t useridnr, u64_t maxmail)
 {
 	int result = 0;
 
-	if (auth_change_mailboxsize(useridnr, maxmail) != 0) {
-		qerrorf("Error: could not change max mail size.\n");
-		result = -1;
+	if (no_to_all) {
+		qprintf("Pretending to change mail quota for user id number [%llu] to [%llu] bytes\n",
+			useridnr, maxmail);
+		return 1;
 	}
 
+	if ((result = auth_change_mailboxsize(useridnr, maxmail)))
+		qerrorf("Error: could not change max mail size.\n");
+
 	return result;
 }
 
@@ -302,6 +306,28 @@ int do_forwards(const char * const alias, const u64_t clientid,
 	char *forward;
 	GList *current_fwds, *matching_fwds, *matching_fwds_del;
 
+	if (no_to_all) {
+		qprintf("Pretending to remove forwards for alias [%s]\n",
+			alias);
+		if (fwds_del) {
+			fwds_del = g_list_first(fwds_del);
+			while (fwds_del) {
+				qprintf("  [%s]\n", (char *)fwds_del->data);
+				fwds_del = g_list_next(fwds_del);
+			}
+		}
+		qprintf("Pretending to add forwards for alias [%s]\n",
+			alias);
+		if (fwds_add) {
+			fwds_add = g_list_first(fwds_add);
+			while (fwds_add) {
+				qprintf("  [%s]\n", (char *)fwds_add->data);
+				fwds_add = g_list_next(fwds_add);
+			}
+		}
+		return 1;
+	}
+
 	current_fwds = auth_get_aliases_ext(alias);
 
 	/* Delete aliases for the user. */
@@ -379,6 +405,28 @@ int do_aliases(const u64_t useridnr,
 	u64_t clientid;
 	GList *current_aliases, *matching_aliases, *matching_alias_del;
 
+	if (no_to_all) {
+		qprintf("Pretending to remove aliases for user id number [%llu]\n",
+			useridnr);
+		if (alias_del) {
+			alias_del = g_list_first(alias_del);
+			while (alias_del) {
+				qprintf("  [%s]\n", (char *)alias_del->data);
+				alias_del = g_list_next(alias_del);
+			}
+		}
+		qprintf("Pretending to add aliases for user id number [%llu]\n",
+			useridnr);
+		if (alias_add) {
+			alias_add = g_list_first(alias_add);
+			while (alias_add) {
+				qprintf("  [%s]\n", (char *)alias_add->data);
+				alias_add = g_list_next(alias_add);
+			}
+		}
+		return 1;
+	}
+
 	auth_getclientid(useridnr, &clientid);
 
 	current_aliases = auth_get_user_aliases(useridnr);
@@ -457,6 +505,12 @@ int do_delete(const u64_t useridnr, const char * const name)
 	int result;
 	GList *aliases = NULL;
 
+	if (no_to_all) {
+		qprintf("Pretending to delete alias [%s] for user id number [%llu]\n",
+			name, useridnr);
+		return 1;
+	}
+
 	qprintf("Deleting aliases for user [%s]...\n", name);
 	aliases = auth_get_user_aliases(useridnr);
 	do_aliases(useridnr, NULL, aliases);
@@ -498,7 +552,7 @@ int do_show(const char * const name)
 			}
 			g_list_foreach(users,(GFunc)g_free,NULL);
 		}
-		g_list_free(users);
+		g_list_free(g_list_first(users));
 
 	} else {
 		if (auth_user_exists(name, &useridnr) == -1) {
@@ -530,8 +584,7 @@ int do_show(const char * const name)
 						break;
 					forwards = g_list_next(forwards);
 				}
-				g_list_foreach(forwards,(GFunc)g_free, NULL);
-				g_list_free(forwards);
+				g_list_destroy(forwards);
 			}
 			
 			userids = g_list_first(userids);
@@ -545,7 +598,7 @@ int do_show(const char * const name)
 						break;
 					userids = g_list_next(userids);
 				}
-				g_list_free(userids);
+				g_list_free(g_list_first(userids));
 			}
 			return 0;
 		}
@@ -578,7 +631,7 @@ int do_show(const char * const name)
 		} else {
 			g_list_append_printf(out,"");
 		}
-		g_list_free(userlist);
+		g_list_free(g_list_first(userlist));
 		s = g_list_join(out,":");
 		qprintf("%s\n", s->str);
 		g_string_free(s,TRUE);
@@ -595,30 +648,45 @@ int do_empty(u64_t useridnr)
 {
 	int result;
 
-	qprintf("Emptying mailbox...");
-	fflush(stdout);
+	if (no_to_all) {
+		u64_t *children;
+		u64_t owner_idnr;
+		unsigned nchildren, i;
+		char mailbox[IMAP_MAX_MAILBOX_NAMELEN];
 
-	result = db_empty_mailbox(useridnr);
-	if (result != 0)
-		qerrorf("Error. Please check the log.\n");
-	else
-		qprintf("Ok.\n");
+		qprintf("You've requested to delete all mailboxes owned by user number [%llu]:\n", useridnr);
+
+		db_findmailbox_by_regex(useridnr, "*", &children, &nchildren, 0);
+		for (i = 0; i < nchildren; i++) {
+			/* Given a list of mailbox id numbers, check if the
+			 * user actually owns the mailbox (because that means
+			 * it is on the hit list for deletion) and then look up
+			 * and print out the name of the mailbox. */
+			db_get_mailbox_owner(children[i], &owner_idnr);
+			if (owner_idnr == useridnr) {
+				db_getmailboxname(children[i], useridnr, mailbox);			
+				qprintf("%s\n", mailbox);
+			}
+		}
+
+		qprintf("please run again with -y to actually perform this action.\n");
+		return 1;
+	}
+//	if (yes_to_all) { // TODO: Require -y before taking such drastic action.
+		qprintf("Emptying mailbox... ");
+		fflush(stdout);
+        
+		result = db_empty_mailbox(useridnr);
+		if (result != 0)
+			qerrorf("Error. Please check the log.\n");
+		else
+			qprintf("Ok.\n");
+//	}
 
 	return result;
 }
 
 
-int is_valid(const char *str)
-{
-	int i;
-
-	for (i = 0; str[i]; i++)
-		if (strchr(ValidChars, str[i]) == NULL)
-			return 0;
-
-	return 1;
-}
-
 /*eddy
   This two function was base from "cpu" by Blake Matheny <matheny@dbaseiv.net>
   bgetpwent : get hash password from /etc/shadow
