@@ -18,12 +18,19 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* $Id: debug.c 2347 2006-11-01 21:35:52Z paul $
+/* 
  *
  * Debugging and memory checking functions */
 
 #include "dbmail.h"
+#if defined(BSD4_4)
+extern char     *__progname;            /* Program name, from crt0. */
+#else
+char            *__progname = NULL;
+#endif
 
+char		hostname[16];
+ 
 /* the debug variables */
 static trace_t TRACE_SYSLOG = TRACE_ERROR;  /* default: errors, warnings, fatals */
 static trace_t TRACE_STDERR = TRACE_FATAL;  /* default: fatal errors only */
@@ -63,14 +70,16 @@ static const char * trace_to_text(trace_t level)
  *
  */
 
-void newtrace(int isnew, trace_t level, const char * module,
+void trace(trace_t level, const char * module,
 		const char * file, const char * function,
 		int line, char *formatstring, ...)
 {
 	va_list argp;
 
 	gchar *message;
-	const char *format = "%s:[%s] %s,%s(+%d): %s";
+	const char *syslog_format = "%s:[%s] %s,%s(+%d): %s";
+	const char *stderr_format = "%s %s %s[%d]: %s:[%s] %s,%s(+%d): %s";
+	static int configured=0;
 	size_t l;
 
 	/* Return now if we're not logging anything. */
@@ -85,11 +94,22 @@ void newtrace(int isnew, trace_t level, const char * module,
 	l = strlen(message);
 	
 	if (level <= TRACE_STDERR) {
-		if (isnew && TRACE_STDERR >= TRACE_DEBUG) {
-			fprintf(stderr, format, trace_to_text(level), module, file, function, line, message);
-		} else {
-			fprintf(stderr, "%s %s", trace_to_text(level), message);
-		}
+		time_t now = time(NULL);
+		struct tm *tmp = localtime(&now);
+		char date[32];
+
+ 		if (! configured) {
+ 			memset(&hostname,'\0',16);
+ 			gethostname(&hostname,16);
+ 			configured=1;
+ 		}
+ 
+ 
+		memset(date,0,sizeof(date));
+		strftime(date,32,"%b %d %H:%M:%S", tmp);
+
+ 		fprintf(stderr, stderr_format, date, hostname, __progname?__progname:"", getpid(), trace_to_text(level), module, file, function, line, message);
+ 
 		if (message[l] != '\n')
 			fprintf(stderr, "\n");
 		fflush(stderr);
@@ -98,20 +118,11 @@ void newtrace(int isnew, trace_t level, const char * module,
 	if (level <= TRACE_SYSLOG) {
 		if (message[l] == '\n')
 			message[l] = '\0';
-		if (level <= TRACE_WARNING) {
-			/* set LOG_ALERT at warnings */
-			if (isnew && TRACE_SYSLOG >= TRACE_DEBUG) {
-				syslog(LOG_ALERT, format, trace_to_text(level), module, file, function, line, message);
-			} else {
-				syslog(LOG_ALERT, "%s %s", trace_to_text(level), message);
-			}
-		} else {
-			if (isnew && TRACE_SYSLOG >= TRACE_DEBUG) {
-				syslog(LOG_NOTICE, format, trace_to_text(level), module, file, function, line, message);
-			} else {
-				syslog(LOG_NOTICE, "%s %s", trace_to_text(level), message);
-			}
-		}
+		/* set LOG_ALERT at warnings */
+		if (level <= TRACE_WARNING)
+			syslog(LOG_ALERT, syslog_format, trace_to_text(level), module, file, function, line, message);
+		else
+			syslog(LOG_NOTICE, syslog_format, trace_to_text(level), module, file, function, line, message);
 	}
 	g_free(message);
 
