@@ -18,8 +18,7 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* $Id: imap4.h 2313 2006-10-21 07:50:51Z paul $
- * 
+/*
  * imap4.h
  */
 
@@ -38,7 +37,7 @@
 /* max number of retries when synchronizing mailbox with dbase */
 #define MAX_RETRIES 20
 
-#define null_free(p) { dm_free(p); p = NULL; }
+#define null_free(p) { g_free(p); p = NULL; }
 
 
 
@@ -50,6 +49,10 @@ typedef struct {
 
 	char partspec[IMAP_MAX_PARTSPEC_LEN];	/* part specifier (i.e. '2.1.3' */
 
+	gchar *hdrnames;
+	gchar *hdrplist;
+
+
 } body_fetch_t;
 
 
