@@ -18,7 +18,7 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* $Id: imapcommands.c 2355 2006-11-05 09:59:43Z paul $
+/* 
  *
  * imapcommands.c
  * 
@@ -32,9 +32,6 @@
 #define _GNU_SOURCE
 #endif
 
-#ifndef MAX_LINESIZE
-#define MAX_LINESIZE 1024
-#endif
 
 #ifndef MAX_RETRIES
 #define MAX_RETRIES 12
@@ -86,9 +83,15 @@ int _ic_capability(struct ImapSession *self)
  */
 int _ic_noop(struct ImapSession *self)
 {
+	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
 	if (!check_state_and_args(self, "NOOP", 0, 0, -1))
 		return 1;	/* error, return */
 
+	if (ud->state == IMAPCS_SELECTED)
+		dbmail_imap_session_mailbox_status(self, TRUE);
+	else
+		TRACE(TRACE_DEBUG,"state: %d", ud->state);
+	
 	dbmail_imap_session_printf(self, "%s OK NOOP completed\r\n", self->tag);
 	return 0;
 }
@@ -137,11 +140,13 @@ int _ic_login(struct ImapSession *self)
 		return 1;
 	
 	create_current_timestring(&timestring);
-	if ((result = dbmail_imap_session_handle_auth(self, self->args[0], self->args[1])))
+	if ((result = dbmail_imap_session_handle_auth(self, self->args[self->args_idx], self->args[self->args_idx+1])))
 		return result;
 	if (imap_before_smtp)
 		db_log_ip(self->ci->ip_src);
 
+	child_reg_connected_user(self->args[self->args_idx]);
+
 	dbmail_imap_session_printf(self, "%s OK LOGIN completed\r\n", self->tag);
 	return 0;
 }
@@ -168,7 +173,7 @@ int _ic_authenticate(struct ImapSession *self)
 	create_current_timestring(&timestring);
 
 	/* check authentication method */
-	if (strcasecmp(self->args[0], "login") != 0) {
+	if (strcasecmp(self->args[self->args_idx], "login") != 0) {
 		dbmail_imap_session_printf(self,
 			"%s NO Invalid authentication mechanism specified\r\n",
 			self->tag);
@@ -225,15 +230,15 @@ int _ic_select(struct ImapSession *self)
 {
 	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
 	u64_t key = 0;
+	u64_t *msn = NULL;
 	int result;
-	unsigned idx = 0;
 	char *mailbox;
 	char permstring[PERMSTRING_SIZE];
 
 	if (!check_state_and_args(self, "SELECT", 1, 1, IMAPCS_AUTHENTICATED))
 		return 1;	/* error, return */
 
-	mailbox = self->args[0];
+	mailbox = self->args[self->args_idx];
 	
 	if ((result = dbmail_imap_session_mailbox_open(self, mailbox))) 
 		return result;
@@ -244,14 +249,10 @@ int _ic_select(struct ImapSession *self)
 	/* show idx of first unseen msg (if present) */
 	if (ud->mailbox.exists) {
 		key = db_first_unseen(ud->mailbox.uid);
-		if (key == (u64_t) (-1)) {
-			dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
-			return -1;
-		}
-		if (binary_search(ud->mailbox.seq_list, ud->mailbox.exists, key, &idx) != -1)
+		if ( (key > 0) && (msn = g_tree_lookup(self->mailbox->ids, &key))) {
 			dbmail_imap_session_printf(self,
-				"* OK [UNSEEN %u] first unseen message\r\n",
-				idx + 1);
+				"* OK [UNSEEN %llu] first unseen message\r\n", *msn);
+		}
 	}
 	/* permission */
 	switch (ud->mailbox.permission) {
@@ -260,11 +261,10 @@ int _ic_select(struct ImapSession *self)
 		break;
 	case IMAPPERM_READWRITE:
 		g_snprintf(permstring, PERMSTRING_SIZE, "READ-WRITE");
-		dbmail_imap_session_mailbox_select_recent(self);
 		break;
 	default:
 		TRACE(TRACE_ERROR, "detected invalid permission mode for mailbox %llu ('%s')",
-		      ud->mailbox.uid, self->args[0]);
+		      ud->mailbox.uid, self->args[self->args_idx]);
 
 		dbmail_imap_session_printf(self,
 			"* BYE fatal: detected invalid mailbox settings\r\n");
@@ -292,7 +292,7 @@ int _ic_examine(struct ImapSession *self)
 	if (!check_state_and_args(self, "EXAMINE", 1, 1, IMAPCS_AUTHENTICATED))
 		return 1;
 
-	mailbox = self->args[0];
+	mailbox = self->args[self->args_idx];
 
 	if ((result = dbmail_imap_session_mailbox_open(self, mailbox)))
 		return result;
@@ -326,7 +326,7 @@ int _ic_create(struct ImapSession *self)
 		return 1;
 
 	/* Create the mailbox and its parents. */
-	result = db_mailbox_create_with_parents(self->args[0], BOX_COMMANDLINE, ud->userid, &mboxid, &message);
+	result = db_mailbox_create_with_parents(self->args[self->args_idx], BOX_COMMANDLINE, ud->userid, &mboxid, &message);
 
 	if (result > 0) {
 		dbmail_imap_session_printf(self, "%s NO %s\r\n", self->tag, message);
@@ -387,12 +387,12 @@ int _ic_delete(struct ImapSession *self)
 		result = db_isselectable(mboxid);
 		if (result == 0) {
 			dbmail_imap_session_printf(self, "%s NO mailbox is non-selectable\r\n", self->tag);
-			dm_free(children);
+			g_free(children);
 			return 1;
 		}
 		if (result == -1) {
 			dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
-			dm_free(children);
+			g_free(children);
 			return -1;	/* fatal */
 		}
 
@@ -403,7 +403,7 @@ int _ic_delete(struct ImapSession *self)
 
 		if (result == -1) {
 			dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
-			dm_free(children);
+			g_free(children);
 			return -1;	/* fatal */
 		}
 
@@ -413,7 +413,7 @@ int _ic_delete(struct ImapSession *self)
 
 		/* ok done */
 		dbmail_imap_session_printf(self, "%s OK DELETE completed\r\n", self->tag);
-		dm_free(children);
+		g_free(children);
 		return 0;
 	}
 
@@ -556,7 +556,7 @@ int _ic_rename(struct ImapSession *self)
 		result = db_getmailboxname(children[i], ud->userid, name);
 		if (result == -1) {
 			dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
-			dm_free(children);
+			g_free(children);
 			return -1;
 		}
 
@@ -564,7 +564,7 @@ int _ic_rename(struct ImapSession *self)
 			/* strange error, let's say its fatal */
 			TRACE(TRACE_ERROR, "mailbox names appear to be corrupted");
 			dbmail_imap_session_printf(self, "* BYE internal error regarding mailbox names\r\n");
-			dm_free(children);
+			g_free(children);
 			return -1;
 		}
 
@@ -573,13 +573,13 @@ int _ic_rename(struct ImapSession *self)
 		result = db_setmailboxname(children[i], newname);
 		if (result == -1) {
 			dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
-			dm_free(children);
+			g_free(children);
 			return -1;
 		}
 			
 	}
 	if (children)
-		dm_free(children);
+		g_free(children);
 
 	/* now replace name */
 	result = db_setmailboxname(mboxid, self->args[1]);
@@ -695,7 +695,7 @@ int _ic_list(struct ImapSession *self)
 	}
 
 	/* check the reference name, should contain only accepted mailboxname chars */
-	for (i = 0, slen = strlen(self->args[0]); self->args[0][i]; i++) {
+	for (i = 0, slen = strlen(AcceptedMailboxnameChars); self->args[0][i]; i++) {
 		if (stridx(AcceptedMailboxnameChars, self->args[0][i]) == slen) {
 			/* wrong char found */
 			dbmail_imap_session_printf(self,
@@ -711,7 +711,7 @@ int _ic_list(struct ImapSession *self)
 	result = db_findmailbox_by_regex(ud->userid, pattern, &children, &nchildren, list_is_lsub);
 	if (result == -1) {
 		dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
-		dm_free(children);
+		g_free(children);
 		g_free(pattern);
 		return -1;
 	}
@@ -719,7 +719,7 @@ int _ic_list(struct ImapSession *self)
 	if (result == 1) {
 		dbmail_imap_session_printf(self, "%s BAD invalid pattern specified\r\n",
 			self->tag);
-		dm_free(children);
+		g_free(children);
 		g_free(pattern);
 		return 1;
 	}
@@ -745,17 +745,16 @@ int _ic_list(struct ImapSession *self)
 		dbmail_imap_session_printf(self, "* %s %s \"%s\" \"%s\"\r\n", thisname, 
 				pstring, MAILBOX_SEPARATOR, mb->name);
 		
-		g_list_foreach(plist,(GFunc)g_free,NULL);
-		g_list_free(plist);
+		g_list_destroy(plist);
 		g_free(pstring);
 	}
 
 
 	if (children)
-		dm_free(children);
+		g_free(children);
 
 	g_free(pattern);
-	dm_free(mb);
+	g_free(mb);
 	dbmail_imap_session_printf(self, "%s OK %s completed\r\n", self->tag, thisname);
 
 	return 0;
@@ -793,17 +792,8 @@ int _ic_status(struct ImapSession *self)
 	gchar *pstring, *astring;
 	
 	
-	/* TODO: check_state_and_args */
-	
-	if (ud->state != IMAPCS_AUTHENTICATED && ud->state != IMAPCS_SELECTED) {
-		dbmail_imap_session_printf(self, "%s BAD STATUS command received in invalid state\r\n", self->tag);
+	if (!check_state_and_args(self, "STATUS", 3, 0, IMAPCS_AUTHENTICATED))
 		return 1;
-	}
-
-	if (!self->args[0] || !self->args[1] || !self->args[2]) {
-		dbmail_imap_session_printf(self, "%s BAD missing argument(s) to STATUS\r\n", self->tag);
-		return 1;
-	}
 
 	if (strcmp(self->args[1], "(") != 0) {
 		dbmail_imap_session_printf(self, "%s BAD argument list should be parenthesed\r\n", self->tag);
@@ -883,7 +873,6 @@ int _ic_status(struct ImapSession *self)
 			dbmail_imap_session_printf(self,
 				"\r\n%s BAD unrecognized option '%s' specified\r\n",
 				self->tag, self->args[i]);
-			dm_free(mb.seq_list);
 			return 1;
 		}
 	}
@@ -895,9 +884,7 @@ int _ic_status(struct ImapSession *self)
 	dbmail_imap_session_printf(self, "%s\r\n", response->str);
 	dbmail_imap_session_printf(self, "%s OK STATUS completed\r\n", self->tag);
 
-	dm_free(mb.seq_list);
-	g_list_foreach(plst,(GFunc)g_free,NULL);
-	g_list_free(plst);
+	g_list_destroy(plst);
 	g_string_free(response,TRUE);
 	g_free(astring);
 	g_free(pstring);
@@ -1129,6 +1116,8 @@ int _ic_check(struct ImapSession *self)
 		return 1;
 	}
 
+	dbmail_imap_session_mailbox_status(self, TRUE);
+
 	dbmail_imap_session_printf(self, "%s OK CHECK completed\r\n", self->tag);
 	return 0;
 }
@@ -1194,14 +1183,13 @@ int _ic_unselect(struct ImapSession *self)
 int _ic_expunge(struct ImapSession *self)
 {
 	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
-	mailbox_t newmailbox;
-	u64_t *msgids;
-	u64_t nmsgs, i;
-	unsigned idx;
+	u64_t *msgids, *msn;
+	u64_t nmsgs, i, uid;
 	int result;
+	
 
 	if (!check_state_and_args(self, "EXPUNGE", 0, 0, IMAPCS_SELECTED))
-		return 1;	/* error, return */
+		return 1; /* error, return */
 
 	if (ud->mailbox.permission != IMAPPERM_READWRITE) {
 		dbmail_imap_session_printf(self,
@@ -1216,60 +1204,38 @@ int _ic_expunge(struct ImapSession *self)
 		return -1;
 	}
 	if (result == 0) {
-		dbmail_imap_session_printf(self,
-			"%s NO you do not have delete rights on this "
-			"mailbox\r\n", self->tag);
+		dbmail_imap_session_printf(self, "%s NO you do not have delete rights on this mailbox\r\n", 
+		self->tag);
 		return 1;
 	}
 
 	/* delete messages */
 	result = db_expunge(ud->mailbox.uid, ud->userid, &msgids, &nmsgs);
 	if (result == -1) {
-		dbmail_imap_session_printf(self, "* BYE dbase/memory error\r\n");
+		dbmail_imap_session_printf(self, "* BYE db error\r\n");
 		return -1;
 	}
-
 	if (result == 1) {
 		dbmail_imap_session_printf(self, "%s OK EXPUNGE completed\r\n", self->tag);
 		return 1;
 	}
-	
-	/* show expunge info */
-	for (i = 0; i < nmsgs; i++) {
-		/* find the message sequence number */
-		binary_search(ud->mailbox.seq_list, ud->mailbox.exists,
-			      msgids[i], &idx);
 
-		dbmail_imap_session_printf(self, "* %u EXPUNGE\r\n", idx + 1);	/* add one: IMAP MSN starts at 1 not zero */
+	for (i=0; i<nmsgs; i++) {
+		uid = msgids[i];
+		msn = g_tree_lookup(self->mailbox->ids, &uid);
+
+		if (! msn) {
+			TRACE(TRACE_DEBUG,"can't find uid [%llu]", uid);
+			break;
+		}
+		dbmail_imap_session_printf(self, "* %llu EXPUNGE\r\n", *msn);
+		dbmail_mailbox_remove_uid(self->mailbox, &uid);
 	}
-	
+		
 	if (msgids)
-		dm_free(msgids);
+		g_free(msgids);
 	msgids = NULL;
 
-	/* update mailbox info */
-
-	memset(&newmailbox, 0, sizeof(newmailbox));
-	newmailbox.uid = ud->mailbox.uid;
-
-	result = db_getmailbox(&newmailbox);
-
-	if (result == -1) {
-		dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
-		dm_free(newmailbox.seq_list);
-		return -1;	/* fatal  */
-	}
-
-	if (newmailbox.exists != ud->mailbox.exists)
-		dbmail_imap_session_printf(self, "* %u EXISTS\r\n", newmailbox.exists);
-
-	if (newmailbox.recent != ud->mailbox.recent)
-		dbmail_imap_session_printf(self, "* %u RECENT\r\n", newmailbox.recent);
-
-	dm_free(ud->mailbox.seq_list);
-	memcpy((void *) &ud->mailbox, (void *) &newmailbox, 
-	       sizeof(newmailbox));
-
 	dbmail_imap_session_printf(self, "%s OK EXPUNGE completed\r\n", self->tag);
 	return 0;
 }
@@ -1287,8 +1253,8 @@ static int sorted_search(struct ImapSession *self, search_order_t order)
 	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
 	struct DbmailMailbox *mb;
 	int result = 0;
-	u64_t idx = 0;
-	gchar *s = NULL, *cmd = NULL;
+	gchar *s = NULL;
+	const gchar *cmd;
 	gboolean sorted;
 
 	if (order == SEARCH_SORTED)
@@ -1318,26 +1284,25 @@ static int sorted_search(struct ImapSession *self, search_order_t order)
 	}
 
 	mb = dbmail_mailbox_new(ud->mailbox.uid);
-
+	switch(order) {
+		case SEARCH_SORTED:
+			cmd = "SORT";
+			break;
+		case SEARCH_UNORDERED:
+			cmd = "SEARCH";
+			break;
+		case SEARCH_THREAD_REFERENCES:
+		case SEARCH_THREAD_ORDEREDSUBJECT:
+			cmd = "THREAD";
+			break;
+		default:// shouldn't happen
+			cmd = "NO";
+			break;
+	}
 	if (g_tree_nnodes(mb->ids) > 0) {
 		dbmail_mailbox_set_uid(mb,self->use_uid);
-		switch(order) {
-			case SEARCH_SORTED:
-				cmd = g_strdup("SORT");
-			break;
-			case SEARCH_UNORDERED:
-				cmd = g_strdup("SEARCH");
-			break;
-			case SEARCH_THREAD_ORDEREDSUBJECT:
-				cmd = g_strdup("THREAD");
-			break;
-			case SEARCH_THREAD_REFERENCES:
-				cmd = g_strdup("THREAD");
-			break;
-		}
 
-
-		if (dbmail_mailbox_build_imap_search(mb, self->args, &idx, order) < 0) {
+		if (dbmail_mailbox_build_imap_search(mb, self->args, &(self->args_idx), order) < 0) {
 			dbmail_imap_session_printf(self, "%s BAD invalid arguments to %s\r\n",
 				self->tag, cmd);
 			return 1;
@@ -1360,15 +1325,16 @@ static int sorted_search(struct ImapSession *self, search_order_t order)
 			break;
 		}
 
-		dbmail_imap_session_printf(self, "* %s %s\r\n", cmd, s?s:"");
-		if (s)
-			g_free(s);
+	}
+
+	if (s) {
+		dbmail_imap_session_printf(self, "* %s %s\r\n", cmd, s);
+		g_free(s);
+	} else {
+		dbmail_imap_session_printf(self, "* %s\r\n", cmd);
 	}
 
 	dbmail_imap_session_printf(self, "%s OK %s completed\r\n", self->tag, cmd);
-	if (cmd)
-		g_free(cmd);
-	
 	dbmail_mailbox_free(mb);
 	
 	return 0;
@@ -1394,6 +1360,38 @@ int _ic_thread(struct ImapSession *self)
 	return 1;
 }
 
+int _dm_imapsession_get_ids(struct ImapSession *self, const char *set)
+{
+	int retry = 2;
+	int result = DM_SUCCESS;
+
+	dbmail_mailbox_set_uid(self->mailbox,self->use_uid);
+
+	if (self->ids) {
+		g_tree_destroy(self->ids);
+		self->ids = NULL;
+	}
+	while (retry > 0) {
+
+		self->ids = dbmail_mailbox_get_set(self->mailbox, set, self->use_uid);
+		if (g_tree_nnodes(self->ids)> 0)
+			break;
+
+		if ((result = dbmail_mailbox_open(self->mailbox))!= DM_SUCCESS)
+			return result;
+		retry--;
+	}
+
+	if (g_tree_nnodes(self->ids)==0) {
+		dbmail_imap_session_printf(self, "%s BAD invalid message range specified\r\n", self->tag);
+		return DM_EGENERAL;
+	}
+
+	return DM_SUCCESS;
+}
+
+
+
 /*
  * _ic_fetch()
  *
@@ -1403,7 +1401,7 @@ int _ic_thread(struct ImapSession *self)
 int _ic_fetch(struct ImapSession *self)
 {
 	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
-	int result, idx;
+	int result, state, setidx;
 
 	if (!check_state_and_args (self, "FETCH", 2, 0, IMAPCS_SELECTED))
 		return 1;
@@ -1423,44 +1421,34 @@ int _ic_fetch(struct ImapSession *self)
 
 	self->fi->getUID = self->use_uid;
 
-	idx = 1;
+	setidx = self->args_idx;
+	self->args_idx++; //skip on past this for the fetch_parse_args coming next...
+
+	state = 1;
 	do {
-		idx = dbmail_imap_session_fetch_parse_args(self, idx);
-		if (idx == -2) {
+		if ( (state = dbmail_imap_session_fetch_parse_args(self)) == -2) {
 			dbmail_imap_session_printf(self, "%s BAD invalid argument list to fetch\r\n", self->tag);
 			return 1;
 		}
-	} while (idx > 0);
+		TRACE(TRACE_DEBUG,"dbmail_imap_session_fetch_parse_args loop idx %llu state %d ", self->args_idx, state);
+		self->args_idx++;
+	} while (state > 0);
 
-	/* reopen the mailbox */
-	dbmail_mailbox_open(self->mailbox);
+	result = DM_SUCCESS;
 
-	if (g_tree_nnodes(self->mailbox->ids) > 0) {
+  	if (g_tree_nnodes(self->mailbox->ids) > 0) {
+ 		if ((result = _dm_imapsession_get_ids(self, self->args[setidx])) == DM_SUCCESS) {
+  			self->ids_list = g_tree_keys(self->ids);
+  			result = dbmail_imap_session_fetch_get_items(self);
+  		}
+	}
 
-		dbmail_mailbox_set_uid(self->mailbox,self->use_uid);
-		
-		self->fetch_ids = dbmail_mailbox_get_set(self->mailbox,self->args[0],self->use_uid);
-		
-		if (g_tree_nnodes(self->fetch_ids)==0) {
-			dbmail_imap_session_printf(self, "%s BAD invalid message range specified\r\n", self->tag);
-			return DM_EGENERAL;
-		}
-			
-		if (dbmail_imap_session_fetch_get_items(self) < 0)
-			return -1;
-		
-		if (self->headers) {
-			g_tree_destroy(self->headers);
-			self->headers = NULL;
-		}
-		if (self->envelopes) {
-			g_tree_destroy(self->envelopes);
-			self->envelopes = NULL;
-		}
-	}	
-	
-	dbmail_imap_session_printf(self, "%s OK %sFETCH completed\r\n", self->tag, self->use_uid ? "UID " : "");
-	return 0;
+	dbmail_imap_session_fetch_free(self);
+
+	if (result == DM_SUCCESS)
+		dbmail_imap_session_printf(self, "%s OK %sFETCH completed\r\n", self->tag, self->use_uid ? "UID " : "");
+
+	return result;
 }
 
 
@@ -1470,98 +1458,143 @@ int _ic_fetch(struct ImapSession *self)
  *
  * alter message-associated data in selected mailbox
  */
+
+static gboolean _do_store(u64_t *id, gpointer UNUSED value, struct ImapSession *self)
+{
+	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
+	cmd_store_t *cmd = (cmd_store_t *)self->cmd;
+
+	u64_t *msn;
+	msginfo_t *msginfo;
+	char *s;
+	int i;
+
+	msginfo = g_tree_lookup(self->msginfo, id);
+	if (! msginfo) {
+		TRACE(TRACE_WARNING, "unable to lookup msginfo struct for [%llu]", *id);
+		return TRUE;
+	}
+
+	msn = g_tree_lookup(self->mailbox->ids, id);
+
+	if (ud->mailbox.permission == IMAPPERM_READWRITE) {
+		if (db_set_msgflag(*id, ud->mailbox.uid, cmd->flaglist, cmd->action) < 0) {
+			dbmail_imap_session_printf(self, "\r\n* BYE internal dbase error\r\n");
+			return TRUE;
+		}
+	}
+
+	for (i = 0; i < IMAP_NFLAGS; i++) {
+		// Skip recent_flag because it is part of the query.
+		if (i == IMAP_FLAG_RECENT)
+			continue;
+		switch (cmd->action) {
+			case IMAPFA_ADD:
+				if (cmd->flaglist[i])
+					msginfo->flags[i] = 1;
+			break;
+			case IMAPFA_REMOVE:
+				if (cmd->flaglist[i]) 
+					msginfo->flags[i] = 0;
+			break;
+			case IMAPFA_REPLACE:
+				if (cmd->flaglist[i]) 
+					msginfo->flags[i] = 1;
+				else
+					msginfo->flags[i] = 0;
+			break;
+		}
+	}
+
+	if (! cmd->silent) {
+		s = imap_flags_as_string(msginfo);
+		dbmail_imap_session_printf(self,"* %llu FETCH (FLAGS %s)\r\n", *msn, s);
+		g_free(s);
+	}
+
+	return FALSE;
+}
+
 int _ic_store(struct ImapSession *self)
 {
 	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
-	char *endptr, *lastchar = NULL;
-	u64_t i, store_start, store_end, seq_max;
-	unsigned fn = 0;
-	int result, j, isfirstout = 0;
-	int be_silent = 0, action = IMAPFA_NONE;
-	int flaglist[IMAP_NFLAGS], msgflags[IMAP_NFLAGS];
-	u64_t thisnum, lo, hi;
+	cmd_store_t cmd;
+	int result, i, j, k;
 
-	memset(flaglist, 0, sizeof(int) * IMAP_NFLAGS);
+	memset(cmd.flaglist, 0, sizeof(int) * IMAP_NFLAGS);
 
-	if (ud->state != IMAPCS_SELECTED) {
-		dbmail_imap_session_printf(self,
-			"%s BAD STORE command received in invalid state\r\n",
-			self->tag);
+	if (!check_state_and_args (self, "STORE", 2, 0, IMAPCS_SELECTED))
 		return 1;
-	}
-
-	if (!self->args[0] || !self->args[1] || !self->args[2]) {
-		dbmail_imap_session_printf(self, "%s BAD missing argument(s) to STORE\r\n",
-			self->tag);
-		return 1;
-	}
 
+	k = self->args_idx;
 	/* multiple flags should be parenthesed */
-	if (self->args[3] && strcmp(self->args[2], "(") != 0) {
+	if (self->args[k+3] && strcmp(self->args[k+2], "(") != 0) {
 		dbmail_imap_session_printf(self, "%s BAD invalid argument(s) to STORE\r\n",
 			self->tag);
 		return 1;
 	}
 
+	cmd.silent = FALSE;
 
 	/* retrieve action type */
-	if (strcasecmp(self->args[1], "flags") == 0)
-		action = IMAPFA_REPLACE;
-	else if (strcasecmp(self->args[1], "flags.silent") == 0) {
-		action = IMAPFA_REPLACE;
-		be_silent = 1;
-	} else if (strcasecmp(self->args[1], "+flags") == 0)
-		action = IMAPFA_ADD;
-	else if (strcasecmp(self->args[1], "+flags.silent") == 0) {
-		action = IMAPFA_ADD;
-		be_silent = 1;
-	} else if (strcasecmp(self->args[1], "-flags") == 0)
-		action = IMAPFA_REMOVE;
-	else if (strcasecmp(self->args[1], "-flags.silent") == 0) {
-		action = IMAPFA_REMOVE;
-		be_silent = 1;
+	if (MATCH(self->args[k+1], "flags"))
+		cmd.action = IMAPFA_REPLACE;
+	else if (MATCH(self->args[k+1], "flags.silent")) {
+		cmd.action = IMAPFA_REPLACE;
+		cmd.silent = TRUE;
+	} else if (MATCH(self->args[k+1], "+flags"))
+		cmd.action = IMAPFA_ADD;
+	else if (MATCH(self->args[k+1], "+flags.silent")) {
+		cmd.action = IMAPFA_ADD;
+		cmd.silent = TRUE;
+	} else if (MATCH(self->args[k+1], "-flags"))
+		cmd.action = IMAPFA_REMOVE;
+	else if (MATCH(self->args[k+1], "-flags.silent")) {
+		cmd.action = IMAPFA_REMOVE;
+		cmd.silent = TRUE;
 	}
 
-	if (action == IMAPFA_NONE) {
-		dbmail_imap_session_printf(self,
-			"%s BAD invalid STORE action specified\r\n", self->tag);
+	if (cmd.action == IMAPFA_NONE) {
+		dbmail_imap_session_printf(self, "%s BAD invalid STORE action specified\r\n", self->tag);
 		return 1;
 	}
 
 	/* now fetch flag list */
-	i = (strcmp(self->args[2], "(") == 0) ? 3 : 2;
+	i = (strcmp(self->args[k+2], "(") == 0) ? 3 : 2;
 
-	for (; self->args[i] && strcmp(self->args[i], ")") != 0; i++) {
-		for (j = 0; j < IMAP_NFLAGS; j++)
-			if (strcasecmp(self->args[i], imap_flag_desc_escaped[j])
-			    == 0) {
-				flaglist[j] = 1;
+	for (; self->args[k+i] && strcmp(self->args[k+i], ")") != 0; i++) {
+		for (j = 0; j < IMAP_NFLAGS; j++) {
+			/* storing the recent flag explicitely is not allowed */
+			if (MATCH(self->args[k+i],"\\Recent")) {
+				j = IMAP_NFLAGS;
 				break;
 			}
+				
+			if (MATCH(self->args[k+i], imap_flag_desc_escaped[j])) {
+				cmd.flaglist[j] = 1;
+				break;
+			}
+		}
 
 		if (j == IMAP_NFLAGS) {
-			dbmail_imap_session_printf(self,
-				"%s BAD invalid flag list to STORE command\r\n",
-				self->tag);
+			dbmail_imap_session_printf(self, "%s BAD invalid flag list to STORE command\r\n", self->tag);
 			return 1;
 		}
 	}
 
-  /** check ACL's for STORE */
-	if (flaglist[IMAP_FLAG_SEEN] == 1) {
+	/** check ACL's for STORE */
+	if (cmd.flaglist[IMAP_FLAG_SEEN] == 1) {
 		result = acl_has_right(&ud->mailbox, ud->userid, ACL_RIGHT_SEEN);
 		if (result < 0) {
 			dbmail_imap_session_printf(self, "* BYE internal database error");
 			return -1;	/* fatal */
 		}
 		if (result == 0) {
-			dbmail_imap_session_printf(self,
-				"%s NO no right to store \\SEEN flag\r\n",
-				self->tag);
+			dbmail_imap_session_printf(self, "%s NO no right to store \\SEEN flag\r\n", self->tag);
 			return 1;
 		}
 	}
-	if (flaglist[IMAP_FLAG_DELETED] == 1) {
+	if (cmd.flaglist[IMAP_FLAG_DELETED] == 1) {
 		result = acl_has_right(&ud->mailbox, ud->userid, ACL_RIGHT_DELETE);
 		if (result < 0) {
 			dbmail_imap_session_printf(self, "* BYE internal database error\r\n");
@@ -1572,212 +1605,43 @@ int _ic_store(struct ImapSession *self)
 			return 1;
 		}
 	}
-	if (flaglist[IMAP_FLAG_ANSWERED] == 1 ||
-	    flaglist[IMAP_FLAG_FLAGGED] == 1 ||
-	    flaglist[IMAP_FLAG_DRAFT] == 1 ||
-	    flaglist[IMAP_FLAG_RECENT] == 1) {
+	if (cmd.flaglist[IMAP_FLAG_ANSWERED] == 1 ||
+	    cmd.flaglist[IMAP_FLAG_FLAGGED] == 1 ||
+	    cmd.flaglist[IMAP_FLAG_DRAFT] == 1) {
 		result = acl_has_right(&ud->mailbox, ud->userid, ACL_RIGHT_WRITE);
 		if (result < 0) {
-			dbmail_imap_session_printf(self, "*BYE internal database error");
+			dbmail_imap_session_printf(self, "* BYE internal database error");
 			return -1;
 		}
 		if (result == 0) {
-			dbmail_imap_session_printf(self, "%s NO no right to store flags",
-				self->tag);
+			dbmail_imap_session_printf(self, "%s NO no right to store flags", self->tag);
 			return 1;
 		}
 	}
 	/* end of ACL checking. If we get here without returning, the user has
 	   the right to store the flags */
 
-	db_getmailbox(&ud->mailbox); // resync mailbox
-	seq_max = (self->use_uid ? (ud->mailbox.msguidnext - 1) : ud->mailbox.exists);
+	self->cmd = &cmd;
 	
-	/* set flags & show if needed */
-	endptr = self->args[0];
-	while (*endptr) {
-		if (endptr != self->args[0])
-			endptr++;	/* skip delimiter */
+	result = DM_SUCCESS;
 
-		store_start = strtoull(endptr, &endptr, 10);
-		
-		if (store_start == 0xffffffff) // outlook's idea of '*'
-			store_start = seq_max;
+  	if (g_tree_nnodes(self->mailbox->ids) > 0) {
+ 		if ((result = _dm_imapsession_get_ids(self, self->args[k])) == DM_SUCCESS) {
+ 			GTree *t;
+ 			t = self->msginfo;
+ 			if ((self->msginfo = dbmail_imap_session_get_msginfo(self, self->mailbox->ids)) == NULL)
+ 				TRACE(TRACE_DEBUG, "unable to retrieve msginfo");
+ 			if(t)
+ 				g_tree_destroy(t);
+ 
+ 			g_tree_foreach(self->ids, (GTraverseFunc) _do_store, self);
+  		}
+  	}	
 
-		if (store_start == 0 || store_start > seq_max) {
-			dbmail_imap_session_printf(self,
-				"%s BAD invalid message range specified\r\n",
-				self->tag);
-			return 1;
-		}
+	if (result == DM_SUCCESS)
+		dbmail_imap_session_printf(self, "%s OK %sSTORE completed\r\n", self->tag, self->use_uid ? "UID " : "");
 
-		switch (*endptr) {
-		case ':':
-			store_end = strtoull(++endptr, &lastchar, 10);
-
-			if (store_end == 0xffffffff) // outlook's idea of '*'
-				store_end = seq_max;
-			
-			endptr = lastchar;
-
-			if (*endptr == '*') {
-				store_end = (self->use_uid ?
-					     (ud->mailbox.msguidnext -
-					      1) : ud->mailbox.exists);
-				endptr++;
-				break;
-			}
-
-			if (store_end == 0 || store_end > seq_max) {
-				dbmail_imap_session_printf(self, "%s BAD invalid message range specified\r\n",
-					self->tag);
-				return 1;
-			}
-
-			if (store_end < store_start) {
-				i = store_start;
-				store_start = store_end;
-				store_end = i;
-			}
-			break;
-
-		case ',':
-		case 0:
-			store_end = store_start;
-			break;
-
-		default:
-			dbmail_imap_session_printf(self,
-				"%s BAD invalid character in message range\r\n",
-				self->tag);
-			return 1;
-		}
-
-		if (!self->use_uid) {
-			store_start--;
-			store_end--;
-		}
-
-		if (store_start == store_end) {
-			thisnum =
-			    (self->use_uid ? store_start : ud->
-			     mailbox.seq_list[store_start]);
-
-			if (self->use_uid) {
-				/* check if the message with this UID belongs to this mailbox */
-				if (binary_search
-				    (ud->mailbox.seq_list,
-				     ud->mailbox.exists, store_start,
-				     &fn) == -1)
-					continue;
-			}
-			if (ud->mailbox.permission == IMAPPERM_READWRITE) {
-				result = db_set_msgflag(thisnum, ud->mailbox.uid, flaglist, action);
-				if (result == -1) {
-					dbmail_imap_session_printf(self, "\r\n* BYE internal dbase error\r\n");
-					return -1;
-				}
-			}
-			if (!be_silent) {
-				result = db_get_msgflag_all(thisnum, ud->mailbox.uid, msgflags);
-				if (result == -1) {
-					dbmail_imap_session_printf(self, "\r\n* BYE internal dbase error\r\n");
-					return -1;
-				}
-
-				dbmail_imap_session_printf(self, "* %llu FETCH (FLAGS (",
-					self->use_uid ? (u64_t) (fn + 1) : store_start + 1);
-
-				for (j = 0, isfirstout = 1;
-				     j < IMAP_NFLAGS; j++) {
-					if (msgflags[j]) {
-						dbmail_imap_session_printf(self, "%s%s",
-							isfirstout ? "" :
-							" ",
-							imap_flag_desc_escaped
-							[j]);
-						if (isfirstout)
-							isfirstout = 0;
-					}
-				}
-
-				dbmail_imap_session_printf(self, "))\r\n");
-			}
-		} else {
-			if (!self->use_uid) {
-				/* find the msgUID's to use */
-				lo = ud->mailbox.seq_list[store_start];
-				hi = ud->mailbox.seq_list[store_end];
-
-			} else {
-				lo = store_start;
-				hi = store_end;
-			}
-
-			if (ud->mailbox.permission == IMAPPERM_READWRITE) {
-				result = db_set_msgflag_range(lo, hi, ud->mailbox.uid, flaglist, action);
-				if (result == -1) {
-					dbmail_imap_session_printf(self, "\r\n* BYE internal dbase error\r\n");
-					return -1;
-				}
-			}
-			if (!be_silent) {
-				for (i = store_start; i <= store_end; i++) {
-					thisnum =
-					    (self->use_uid ? i :
-					     ud->mailbox.seq_list[i]);
-
-					if (self->use_uid) {
-						/* check if the message with this UID belongs
-						   to this mailbox */
-						if (binary_search
-						    (ud->mailbox.seq_list,
-						     ud->mailbox.exists, i,
-						     &fn) == -1)
-							continue;
-					}
-
-					result =
-					    db_get_msgflag_all(thisnum,
-							       ud->mailbox.
-							       uid,
-							       msgflags);
-					if (result == -1) {
-						dbmail_imap_session_printf(self,
-							"\r\n* BYE internal dbase error\r\n");
-						return -1;
-					}
-
-					dbmail_imap_session_printf(self,
-						"* %llu FETCH (FLAGS (",
-						self->use_uid
-						? (u64_t) (fn + 1) : i +
-						1);
-
-					for (j = 0, isfirstout = 1;
-					     j < IMAP_NFLAGS; j++) {
-						if (msgflags[j]) {
-							dbmail_imap_session_printf(self,
-								"%s%s",
-								isfirstout
-								? "" : " ",
-								imap_flag_desc_escaped
-								[j]);
-							if (isfirstout)
-								isfirstout
-								    = 0;
-						}
-					}
-
-					dbmail_imap_session_printf(self, "))\r\n");
-				}
-			}
-		}
-	}
-
-	dbmail_imap_session_printf(self, "%s OK %sSTORE completed\r\n", self->tag,
-		self->use_uid ? "UID " : "");
-	return 0;
+	return result;
 }
 
 
@@ -1786,24 +1650,44 @@ int _ic_store(struct ImapSession *self)
  *
  * copy a message to another mailbox
  */
+
+static gboolean _do_copy(u64_t *id, gpointer UNUSED value, struct ImapSession *self)
+{
+	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
+	cmd_copy_t *cmd = (cmd_copy_t *)self->cmd;
+	u64_t newid;
+	int result;
+
+	result = db_copymsg(*id, cmd->mailbox_id, ud->userid, &newid);
+	if (result == -1) {
+		dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
+		db_rollback_transaction();
+		return TRUE;
+	}
+	if (result == -2) {
+		dbmail_imap_session_printf(self, "%s NO quotum would exceed\r\n", self->tag);
+		db_rollback_transaction();
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 int _ic_copy(struct ImapSession *self)
 {
 	imap_userdata_t *ud = (imap_userdata_t *) self->ci->userData;
-	u64_t i, copy_start, copy_end;
-	unsigned fn;
-	u64_t destmboxid, thisnum;
+	u64_t destmboxid;
 	int result;
-	u64_t new_msgid;
-	char *endptr, *lastchar = NULL;
 	mailbox_t destmbox;
+	cmd_copy_t cmd;
 	
-	bzero(&destmbox, sizeof(destmbox));
+	memset(&destmbox, 0, sizeof(destmbox));
 
 	if (!check_state_and_args(self, "COPY", 2, 2, IMAPCS_SELECTED))
 		return 1;	/* error, return */
 
 	/* check if destination mailbox exists */
-	if (db_findmailbox(self->args[1], ud->userid, &destmboxid) == -1) {
+	if (db_findmailbox(self->args[self->args_idx+1], ud->userid, &destmboxid) == -1) {
 		dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
 		return -1;	/* fatal */
 	}
@@ -1838,106 +1722,21 @@ int _ic_copy(struct ImapSession *self)
 		return 1;
 	}
 
-	db_getmailbox(&ud->mailbox); // resync mailbox
-	
-	/* ok copy msgs */
-	endptr = self->args[0];
-	
+	cmd.mailbox_id = destmboxid;
+	self->cmd = &cmd;
+
 	if (db_begin_transaction() < 0)
 		return -1;
-	
-	while (*endptr) {
-		if (endptr != self->args[0])
-			endptr++;	/* skip delimiter */
 
-		copy_start = strtoull(endptr, &lastchar, 10);
-		endptr = lastchar;
-
-		if (copy_start == 0 || copy_start >
-		    (self->use_uid ? (ud->mailbox.msguidnext - 1) :
-		     ud->mailbox.exists)) {
-			dbmail_imap_session_printf(self,
-				"%s BAD invalid message range specified\r\n",
-				self->tag);
-			db_rollback_transaction();
-			return 1;
-		}
-
-		switch (*endptr) {
-		case ':':
-			copy_end = strtoull(++endptr, &lastchar, 10);
-			endptr = lastchar;
-
-			if (*endptr == '*') {
-				copy_end = (self->use_uid ?
-					    (ud->mailbox.msguidnext -
-					     1) : ud->mailbox.exists);
-				endptr++;
-				break;
-			}
-
-			if (copy_end == 0 || copy_end >
-			    (self->use_uid
-			     ? (ud->mailbox.msguidnext -
-				1) : ud->mailbox.exists)) {
-				dbmail_imap_session_printf(self,
-					"%s BAD invalid message range specified\r\n",
-					self->tag);
-				db_rollback_transaction();
-				return 1;
-			}
-
-			if (copy_end < copy_start) {
-				i = copy_start;
-				copy_start = copy_end;
-				copy_end = i;
-			}
-			break;
-
-		case ',':
-		case 0:
-			copy_end = copy_start;
-			break;
-
-		default:
-			dbmail_imap_session_printf(self,
-				"%s BAD invalid character in message range\r\n",
-				self->tag);
-			db_rollback_transaction();
-			return 1;
-		}
-
-		if (!self->use_uid) {
-			copy_start--;
-			copy_end--;
-		}
-
-		for (i = copy_start; i <= copy_end; i++) {
-			thisnum =
-			    (self->use_uid ? i : ud->mailbox.
-			     seq_list[i]);
-
-			if (self->use_uid) {
-				/* check if the message with this UID belongs to this mailbox */
-				if (binary_search
-				    (ud->mailbox.seq_list,
-				     ud->mailbox.exists, i, &fn) == -1)
-					continue;
-			}
-
-			result = db_copymsg(thisnum, destmboxid, ud->userid, &new_msgid);
-			if (result == -1) {
-				dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
-				db_rollback_transaction();
-				return -1;
-			}
-			if (result == -2) {
-				dbmail_imap_session_printf(self, "%s NO quotum would exceed\r\n", self->tag);
-				db_rollback_transaction();
-				return 1;
-			}
-		}
-	}
+	if (g_tree_nnodes(self->mailbox->ids) > 0) {
+  
+ 		if ((_dm_imapsession_get_ids(self, self->args[self->args_idx]) == DM_SUCCESS)) {
+ 			g_tree_foreach(self->ids, (GTraverseFunc) _do_copy, self);
+ 		} else {
+ 			db_rollback_transaction();
+  			return DM_EGENERAL;
+  		}
+  	}	
 
 	if (db_commit_transaction() < 0)
 		return -1;
@@ -1974,23 +1773,23 @@ int _ic_uid(struct ImapSession *self)
 	self->use_uid = 1;	/* set global var to make clear we will be using UID's */
 	
 	/* ACL rights for UID are handled by the other functions called below */
-	if (MATCH(self->args[0], "fetch")) {
-		self->args++;
+	if (MATCH(self->args[self->args_idx], "fetch")) {
+		self->args_idx++; 
 		result = _ic_fetch(self);
-	} else if (MATCH(self->args[0], "copy")) {
-		self->args++;
+	} else if (MATCH(self->args[self->args_idx], "copy")) {
+		self->args_idx++;
 		result = _ic_copy(self);
-	} else if (MATCH(self->args[0], "store")) {
-		self->args++;
+	} else if (MATCH(self->args[self->args_idx], "store")) {
+		self->args_idx++;
 		result = _ic_store(self);
-	} else if (MATCH(self->args[0], "search")) {
-		self->args++;
+	} else if (MATCH(self->args[self->args_idx], "search")) {
+		self->args_idx++;
 		result = _ic_search(self);
-	} else if (MATCH(self->args[0], "sort")) {
-		self->args++;
+	} else if (MATCH(self->args[self->args_idx], "sort")) {
+		self->args_idx++;
 		result = _ic_sort(self);
-	} else if (MATCH(self->args[0], "thread")) {
-		self->args++;
+	} else if (MATCH(self->args[self->args_idx], "thread")) {
+		self->args_idx++;
 		result = _ic_thread(self);
 	} else {
 		dbmail_imap_session_printf(self, "%s BAD invalid UID command\r\n", self->tag);
@@ -2044,7 +1843,7 @@ int _ic_getquotaroot(struct ImapSession *self)
 	if (!check_state_and_args(self, "GETQUOTAROOT", 1, 1, IMAPCS_AUTHENTICATED))
 		return 1;	/* error, return */
 
-	root = quota_get_quotaroot(ud->userid, self->args[0], &errormsg);
+	root = quota_get_quotaroot(ud->userid, self->args[self->args_idx], &errormsg);
 	if (root == NULL) {
 		dbmail_imap_session_printf(self, "%s NO %s\r\n", self->tag, errormsg);
 		return 1;
@@ -2056,7 +1855,7 @@ int _ic_getquotaroot(struct ImapSession *self)
 		return 1;
 	}
 
-	dbmail_imap_session_printf(self, "* QUOTAROOT \"%s\" \"%s\"\r\n", self->args[0],
+	dbmail_imap_session_printf(self, "* QUOTAROOT \"%s\" \"%s\"\r\n", self->args[self->args_idx],
 		quota->root);
 	send_quota(self, quota);
 	quota_free(quota);
@@ -2079,7 +1878,7 @@ int _ic_getquota(struct ImapSession *self)
 	if (!check_state_and_args(self, "GETQUOTA", 1, 1, IMAPCS_AUTHENTICATED))
 		return 1;	/* error, return */
 
-	quota = quota_get_quota(ud->userid, self->args[0], &errormsg);
+	quota = quota_get_quota(ud->userid, self->args[self->args_idx], &errormsg);
 	if (quota == NULL) {
 		dbmail_imap_session_printf(self, "%s NO %s\r\n", self->tag, errormsg);
 		return 1;
@@ -2124,7 +1923,7 @@ int _ic_setacl(struct ImapSession *self)
 	if (!check_state_and_args(self, "SETACL", 3, 3, IMAPCS_AUTHENTICATED))
 		return 1;
 
-	result = imap_acl_pre_administer(self->args[0], self->args[1], ud->userid,
+	result = imap_acl_pre_administer(self->args[self->args_idx], self->args[self->args_idx+1], ud->userid,
 					 &mboxid, &targetuserid);
 	if (result == -1) {
 		dbmail_imap_session_printf(self, "* BYE internal database error\r\n");
@@ -2142,7 +1941,7 @@ int _ic_setacl(struct ImapSession *self)
 		return 1;
 	}
 	// set the new acl
-	if (acl_set_rights(targetuserid, mboxid, self->args[2]) < 0) {
+	if (acl_set_rights(targetuserid, mboxid, self->args[self->args_idx+2]) < 0) {
 		dbmail_imap_session_printf(self, "* BYE internal database error\r\n");
 		return -1;
 	}
@@ -2163,7 +1962,7 @@ int _ic_deleteacl(struct ImapSession *self)
 	if (!check_state_and_args(self, "DELETEACL", 2, 2, IMAPCS_AUTHENTICATED))
 		return 1;
 
-	if (imap_acl_pre_administer(self->args[0], self->args[1], ud->userid,
+	if (imap_acl_pre_administer(self->args[self->args_idx], self->args[self->args_idx+1], ud->userid,
 				    &mboxid, &targetuserid) == -1) {
 		dbmail_imap_session_printf(self, "* BYE internal dbase error\r\n");
 		return -1;
@@ -2198,7 +1997,7 @@ int _ic_getacl(struct ImapSession *self)
 	if (!check_state_and_args(self, "GETACL", 1, 1, IMAPCS_AUTHENTICATED))
 		return 1;
 
-	result = db_findmailbox(self->args[0], ud->userid, &mboxid);
+	result = db_findmailbox(self->args[self->args_idx], ud->userid, &mboxid);
 	if (result == -1) {
 		dbmail_imap_session_printf(self, "* BYE internal database error\r\n");
 		return -1;
@@ -2213,8 +2012,8 @@ int _ic_getacl(struct ImapSession *self)
 		return -1;
 	}
 
-	dbmail_imap_session_printf(self, "* ACL \"%s\" %s\r\n", self->args[0], acl_string);
-	dm_free(acl_string);
+	dbmail_imap_session_printf(self, "* ACL \"%s\" %s\r\n", self->args[self->args_idx], acl_string);
+	g_free(acl_string);
 	dbmail_imap_session_printf(self, "%s OK GETACL completed\r\n", self->tag);
 	return 0;
 }
@@ -2232,7 +2031,7 @@ int _ic_listrights(struct ImapSession *self)
 	if (!check_state_and_args(self, "LISTRIGHTS", 2, 2, IMAPCS_AUTHENTICATED))
 		return 1;
 
-	result = imap_acl_pre_administer(self->args[0], self->args[1], ud->userid,
+	result = imap_acl_pre_administer(self->args[self->args_idx], self->args[self->args_idx+1], ud->userid,
 					 &mboxid, &targetuserid);
 	if (result == -1) {
 		dbmail_imap_session_printf(self, "* BYE internal database error\r\n");
@@ -2259,9 +2058,9 @@ int _ic_listrights(struct ImapSession *self)
 	}
 
 	dbmail_imap_session_printf(self, "* LISTRIGHTS \"%s\" %s %s\r\n",
-		self->args[0], self->args[1], listrights_string);
+		self->args[self->args_idx], self->args[self->args_idx+1], listrights_string);
 	dbmail_imap_session_printf(self, "%s OK LISTRIGHTS completed\r\n", self->tag);
-	dm_free(listrights_string);
+	g_free(listrights_string);
 	return 0;
 }
 
@@ -2276,7 +2075,7 @@ int _ic_myrights(struct ImapSession *self)
 	if (!check_state_and_args(self, "LISTRIGHTS", 1, 1, IMAPCS_AUTHENTICATED))
 		return 1;
 
-	result = db_findmailbox(self->args[0], ud->userid, &mboxid);
+	result = db_findmailbox(self->args[self->args_idx], ud->userid, &mboxid);
 	if (result == -1) {
 		dbmail_imap_session_printf(self, "* BYE internal database error\r\n");
 		return -1;
@@ -2292,9 +2091,9 @@ int _ic_myrights(struct ImapSession *self)
 		return -1;
 	}
 
-	dbmail_imap_session_printf(self, "* MYRIGHTS \"%s\" %s\r\n", self->args[0],
+	dbmail_imap_session_printf(self, "* MYRIGHTS \"%s\" %s\r\n", self->args[self->args_idx],
 		myrights_string);
-	dm_free(myrights_string);
+	g_free(myrights_string);
 	dbmail_imap_session_printf(self, "%s OK MYRIGHTS complete\r\n", self->tag);
 	return 0;
 }
