@@ -19,7 +19,7 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* $Id: imapd.c 2259 2006-09-08 19:48:57Z aaron $
+/* 
  *
  * imapd.c
  * 
@@ -37,9 +37,11 @@ int main(int argc, char *argv[])
 	serverConfig_t config;
 	int result;
 		
-	g_mime_init(0);
 	openlog(PNAME, LOG_PID, LOG_MAIL);
 
+       //fixes valgrind Conditional jump or move depends on uninitialised value(s)
+       memset(&config, 0, sizeof(serverConfig_t));
+
 	result = serverparent_getopt(&config, "IMAP", argc, argv);
 	if (result == -1)
 		goto shutdown;
@@ -52,14 +54,12 @@ int main(int argc, char *argv[])
 
 	/* These are in imap4.h */
 	config.ClientHandler = IMAPClientHandler;
-	config.timeoutMsg = IMAP_TIMEOUT_MSG;
 
 	imap_before_smtp = config.service_before_smtp;
 
 	result = serverparent_mainloop(&config, "IMAP", "dbmail-imapd");
 	
 shutdown:
-	g_mime_shutdown();
 	config_free();
 
 	TRACE(TRACE_INFO, "exit");
