@@ -18,7 +18,7 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/*	$Id: misc.c 2371 2006-11-15 15:55:46Z paul $
+/*	
  *
  *	Miscelaneous functions */
 
@@ -30,7 +30,7 @@
 const char AcceptedMailboxnameChars[] =
     "abcdefghijklmnopqrstuvwxyz"
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-    "0123456789-=/ _.&,+@()[]'";
+    "0123456789-=/ _.&,+@()[]'#";
 
 /**
  * abbreviated names of the months
@@ -50,6 +50,8 @@ char _sqldate[SQL_INTERNALDATE_LEN + 1] = SQL_STANDARD_DATE;
 
 const int month_len[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 
+extern const char *imap_flag_desc_escaped[];
+
 #undef max
 #define max(x,y) ( (x) > (y) ? (x) : (y) )
 
@@ -107,7 +109,7 @@ void create_unique_id(char *target, u64_t message_idnr)
 	md5_str = dm_md5((unsigned char *)target);
 	snprintf(target, UID_SIZE, "%s", md5_str);
 	TRACE(TRACE_DEBUG, "created: %s", target);
-	dm_free(md5_str);
+	g_free(md5_str);
 	g_free(a_message_idnr);
 	g_free(a_rand);
 }
@@ -152,7 +154,7 @@ char *mailbox_add_namespace(const char *mailbox_name, u64_t owner_idnr,
 	else
 		g_string_printf(t, "%s%s%s%s%s", NAMESPACE_USER, MAILBOX_SEPARATOR, owner, 
 				MAILBOX_SEPARATOR, mailbox_name);
-	dm_free(owner);
+	g_free(owner);
 	
 	fq = t->str;
 	g_string_free(t,FALSE);
@@ -166,52 +168,91 @@ char *mailbox_add_namespace(const char *mailbox_name, u64_t owner_idnr,
 const char *mailbox_remove_namespace(const char *fq_name,
 		char **namespace, char **username)
 {
-	char *temp, *user;
-	size_t ns_user_len;
-	size_t ns_publ_len;
+	const char *temp = NULL, *user = NULL, *mbox = NULL;
+	static size_t ns_user_len = 0;
+	static size_t ns_publ_len = 0;
 	size_t fq_name_len;
 
+ 	if(ns_user_len==0) {
+ 		ns_user_len = strlen(NAMESPACE_USER);
+ 		ns_publ_len = strlen(NAMESPACE_PUBLIC);
+ 	}
+
 	if (username) *username = NULL;
 	if (namespace) *namespace = NULL;
 
 	fq_name_len = strlen(fq_name);
-	ns_user_len = strlen(NAMESPACE_USER);
-	ns_publ_len = strlen(NAMESPACE_PUBLIC);
 
 	// i.e. '#Users/someuser/foldername'
+	// fail on '#Users*' and '#Users%'
+	// assume a slash in '#Users/foo*' and '#Users/foo%' like this '#Users/foo/*'
 	if (fq_name_len >= ns_user_len && strncasecmp(fq_name, NAMESPACE_USER, ns_user_len) == 0) {
 		if (namespace) *namespace = NAMESPACE_USER;
-		user = strstr(fq_name, MAILBOX_SEPARATOR);
-		if (user == NULL || strlen(user) <= 1) {
-			TRACE(TRACE_MESSAGE, "illegal mailbox name");
+
+		int end = 0, err = 0, slash = 0;
+		// We'll use a simple state machine to parse through this.
+		for (temp = &fq_name[ns_user_len]; !end && !err; temp++) {
+			switch (*temp) {
+			case '/':
+				if (!user) {
+					user = temp + 1;
+				} else if (user && !mbox) {
+					mbox = temp + 1;
+					slash = 1;
+				} else if (user && mbox) {
+					end = 1;
+				}
+				break;
+			case '*':
+				if (!user)
+					err = 1;
+				mbox = temp;
+				break;
+			case '%':
+				if (!user)
+					err = 1;
+				mbox = temp;
+				break;
+			case '\0':
+				if (!user)
+					err = 1;
+				end = 1;
+				break;
+			}
+		}
+
+		if (err) {
+			TRACE(TRACE_MESSAGE, "Illegal mailbox name");
 			return NULL;
 		}
-		temp = strstr(&user[1], MAILBOX_SEPARATOR);
-		if (temp == NULL || strlen(temp) <= 1) {
-			TRACE(TRACE_MESSAGE, "illegal mailbox name");
+
+		if (!user || user + slash == mbox) {
+			TRACE(TRACE_DEBUG, "Username not found");
 			return NULL;
 		}
-		if (user >= temp) {
-			TRACE(TRACE_DEBUG, "Username not found.");
+
+		if (!mbox) {
+			TRACE(TRACE_DEBUG, "Mailbox not found");
 			return NULL;
-		} else {
-			TRACE(TRACE_DEBUG, "Copying out username [%s] of length [%u]",
-				&user[1], temp - user - 1);
-			if (username) *username = g_strndup(&user[1], temp - user - 1);
 		}
-		return &temp[1];
+
+		TRACE(TRACE_DEBUG, "Copying out username [%s] of length [%zu]",
+			user, (size_t)(mbox - user - slash));
+		if (username) *username = g_strndup(user, mbox - user - slash);
+
+		return mbox;
 	}
 	
 	// i.e. '#Public/foldername'
+	// accept #Public* and #Public% also
 	if (fq_name_len >= ns_publ_len && strncasecmp(fq_name, NAMESPACE_PUBLIC, ns_publ_len) == 0) {
 		if (namespace) *namespace = NAMESPACE_PUBLIC;
-		temp = strstr(fq_name, MAILBOX_SEPARATOR);
-		if (temp == NULL || strlen(temp) <= 1) {
-			TRACE(TRACE_MESSAGE, "illegal mailbox name");
-			return NULL;
-		}
 		if (username) *username = g_strdup(PUBLIC_FOLDER_USER);
-		return &temp[1];
+		// Drop the slash between the namespace and the mailbox spec
+		if (fq_name[ns_publ_len] == '/')
+			return &fq_name[ns_publ_len+1]; 
+		// But if the slash wasn't there, it means we have #Public*, and that's OK.
+		return &fq_name[ns_publ_len]; 
 	}
 	
 	return fq_name;
@@ -248,7 +289,7 @@ int read_from_stream(FILE * instream, char **m_buf, int maxlen)
 
 	/* Allocate a zero length string on length 0. */
 	if (maxlen == 0) {
-		*m_buf = dm_strdup("");
+		*m_buf = g_strdup("");
 		return 0;
 	}
 
@@ -321,12 +362,12 @@ int find_bounded(const char * const value, char left, char right,
 			tmpleft++;
 
 		tmplen = tmpright - tmpleft;
-		*retchar = dm_malloc(sizeof(char) * (tmplen + 1));
+		*retchar = g_new0(char, tmplen + 1);
 		if (*retchar == NULL) {
 			*retchar = NULL;
 			*retsize = 0;
 			*retlast = 0;
-			TRACE(TRACE_INFO, "Found [%s] of length [%zd] between '%c' and '%c' so next skip [%zd]", *retchar, *retsize,
+			TRACE(TRACE_INFO, "Found [%s] of length [%zu] between '%c' and '%c' so next skip [%zu]", *retchar, *retsize,
 			      left, right, *retlast);
 			return -2;
 		}
@@ -334,7 +375,7 @@ int find_bounded(const char * const value, char left, char right,
 		(*retchar)[tmplen] = '\0';
 		*retsize = tmplen;
 		*retlast = tmpright - value;
-		TRACE(TRACE_INFO, "Found [%s] of length [%zd] between '%c' and '%c' so next skip [%zd]", *retchar, *retsize, left,
+		TRACE(TRACE_INFO, "Found [%s] of length [%zu] between '%c' and '%c' so next skip [%zu]", *retchar, *retsize, left,
 		      right, *retlast);
 		return 0;
 	}
@@ -352,13 +393,13 @@ int zap_between(const char * const instring, signed char left, signed char right
 
 	// Should we clip the left char, too?
 	if (left < 0) {
-		left = 0 - left;
+		left = (signed char)(0 - left);
 		clipleft = 1;
 	}
 
 	// Should we clip the right char, too?
 	if (right < 0) {
-		right = 0 - right;
+		right = (signed char)(0 - right);
 		clipright = 1;
 	}
 
@@ -594,13 +635,19 @@ static void _strip_sub_leader(char *subject)
 	_strip_refwd(subject);
 }
 
-void dm_base_subject(char *subject)
+char * dm_base_subject(const char *subject)
 {
 	unsigned offset, len, olen;
 	char *tmp, *saved;
 	
-	tmp = g_mime_utils_header_decode_text((unsigned char *)subject);
+	// we expect utf-8 or 7-bit data
+	if (subject == NULL) return NULL;
+	if (g_mime_utils_text_is_8bit((unsigned char *)subject, strlen(subject))) 
+		tmp = g_strdup(subject);
+	else 
+		tmp = g_mime_utils_header_decode_text((unsigned char *)subject);
 	saved = tmp;
+	
 	dm_pack_spaces(tmp);
 	g_strstrip(tmp);
 	while (1==1) {
@@ -630,8 +677,10 @@ void dm_base_subject(char *subject)
 		if (strlen(tmp)==olen)
 			break;
 	}
-	strncpy(subject,tmp,strlen(subject)+1);
+	tmp = g_strdup(tmp);
 	g_free(saved);
+	
+	return tmp;
 }
 
 /* 
@@ -690,9 +739,8 @@ int listex_match(const char *p, const char *s,
 			/* fall; try regular search */
 		}
 
-		if ((flags & LISTEX_NOCASE && tolower(((unsigned int)*p))
-					== tolower(((unsigned int)*s)))
-		|| (*p == *s)) {
+		if ( (*p == *s)||
+		((flags & LISTEX_NOCASE) && (tolower(*p) == tolower(*s)))) {
 			p8=(((unsigned char)*p) > 0xC0);
 			p++; s++;
 		} else {
@@ -714,14 +762,13 @@ u64_t dm_getguid(unsigned int serverid)
         if (gettimeofday(&tv,NULL))
                 return 0;
 
-        snprintf(s,30,"%ld%06ld%02d", tv.tv_sec, tv.tv_usec,serverid);
+        snprintf(s,30,"%ld%06ld%02u", tv.tv_sec, tv.tv_usec,serverid);
         return (u64_t)strtoll(s,NULL,10);
 }
 
 sa_family_t dm_get_client_sockaddr(clientinfo_t *ci, struct sockaddr *saddr)
 {
-	int maxsocklen = 128; /* ref. UNP */
-	
+	#define maxsocklen	128
 	union {
 		struct sockaddr sa;
 		char data[maxsocklen];
@@ -743,6 +790,9 @@ int dm_sock_score(const char *base, const char *test)
 	int result = 0;
 	char *t;
 
+	if ((! base) || (! test))
+		return 0;
+		
 	t = strstr(base,"unix:");
 	if (t==base) {
 		base = strstr(base,":");
@@ -781,7 +831,7 @@ static int socket_match(const char *base, const char *test)
 
 int dm_sock_compare(const char *clientsock, const char *sock_allow, const char *sock_deny) 
 {
-	int result;
+	int result = DM_EGENERAL;
 	assert(clientsock);
 	
 	if ( (strlen(sock_allow) == 0) && (strlen(sock_deny) == 0) ) {
@@ -790,8 +840,6 @@ int dm_sock_compare(const char *clientsock, const char *sock_allow, const char *
 		result = DM_EGENERAL;
 	} else if (strlen(sock_allow) > 0  && socket_match(sock_allow, clientsock)==0) {
 		result = DM_SUCCESS;
-	} else {
-		result = DM_EGENERAL;
 	}
 
 	TRACE(TRACE_DEBUG, "clientsock [%s] sock_allow[%s], sock_deny [%s] => [%d]", clientsock, sock_allow, sock_deny, result);
@@ -869,16 +917,22 @@ int checkmailboxname(const char *s)
  * 01234567890
  * month three-letter specifier
  */
+ 
+// Define len if "01-Jan-1970" string
+#define STRLEN_MINDATA	11
+
 int check_date(const char *date)
 {
 	char sub[4];
-	int days, i, j;
+	int days, i, j=1;
+	size_t l;
 
-	if (strlen(date) != strlen("01-Jan-1970")
-	    && strlen(date) != strlen("1-Jan-1970"))
+	l = strlen(date);
+
+	if (l != STRLEN_MINDATA && l != STRLEN_MINDATA-1)
 		return 0;
 
-	j = (strlen(date) == strlen("1-Jan-1970")) ? 1 : 0;
+	j = (l==STRLEN_MINDATA) ? 0 : 1;
 
 	if (date[2 - j] != '-' || date[6 - j] != '-')
 		return 0;
@@ -1051,7 +1105,10 @@ int num_from_imapdate(const char *date)
 
 void g_list_destroy(GList *l)
 {
+	l = g_list_first(l);
 	g_list_foreach(l,(GFunc)g_free,NULL);
+
+	l = g_list_first(l);
 	g_list_free(l);
 }
 
@@ -1198,13 +1255,21 @@ int g_tree_merge(GTree *a, GTree *b, int condition)
 				
 				keys = g_list_next(keys);
 			}
+
+			keys = g_list_first(keys);
+			g_list_free(keys);
+
 			break;
 	}
 
-	TRACE(TRACE_DEBUG,"a[%d] [%s] b[%d] -> a[%d]", 
-			alen, type, blen, 
+	TRACE(TRACE_DEBUG,"(%p) (%p): a[%d] [%s] b[%d] -> a[%d]", 
+			a, b, alen, type, blen, 
 			g_tree_nnodes(a));
 
+	merger->list = g_list_first(merger->list);
+	g_list_free(merger->list);
+
+
 	g_free(merger);
 	g_free(type);
 
@@ -1376,8 +1441,7 @@ char *dbmail_imap_plist_as_string(GList * list)
 
 void dbmail_imap_plist_free(GList *l)
 {
-	g_list_foreach(l, (GFunc)g_free, NULL);
-	g_list_free(l);
+	g_list_destroy(l);
 }
 
 /* 
@@ -1387,8 +1451,8 @@ void dbmail_imap_plist_free(GList *l)
 char *dbmail_imap_astring_as_string(const char *s)
 {
 	int i;
-	char *r;
-	char *t, *l = NULL;
+	const char *p;
+	char *r, *t, *l = NULL;
 	char first, last, penult = '\\';
 
 	if (! s)
@@ -1408,7 +1472,11 @@ char *dbmail_imap_astring_as_string(const char *s)
 	
 	for (i=0; l[i]; i++) { 
 		if ((l[i] & 0x80) || (l[i] == '\r') || (l[i] == '\n') || (l[i] == '"') || (l[i] == '\\')) {
-			r = g_strdup_printf("{%lu}\r\n%s", (unsigned long) strlen(l), l);
+			if ((l[i] == '"') && (i>0) && (l[i-1] != '\\'))
+				p = s;
+			else
+				p = l;
+			r = g_strdup_printf("{%lu}\r\n%s", (unsigned long) strlen(p), p);
 			g_free(t);
 			return r;
 		}
@@ -1446,8 +1514,7 @@ static GList * imap_append_hash_as_string(GList *list, GHashTable *hash)
 		list = g_list_append_printf(list, "%s", s);
 		g_free(s);
 		
-		g_list_foreach(l,(GFunc)g_free,NULL);
-		g_list_free(l);
+		g_list_destroy(l);
 	} else {
 		list = g_list_append_printf(list, "NIL");
 	}
@@ -1472,8 +1539,7 @@ static GList * imap_append_disposition_as_string(GList *list, GMimeObject *part)
 		list = g_list_append_printf(list,"%s",result);
 		g_free(result);
 
-		g_list_foreach(t,(GFunc)g_free,NULL);
-		g_list_free(t);
+		g_list_destroy(t);
 		g_mime_disposition_destroy(disposition);
 	} else {
 		list = g_list_append_printf(list,"NIL");
@@ -1623,20 +1689,19 @@ void _structure_part_multipart(GMimeObject *part, gpointer data, gboolean extens
 		
 		alist = g_list_append(alist,s->str);
 
-		g_list_foreach(list,(GFunc)g_free,NULL);
-		g_list_free(list);
+		g_list_destroy(list);
 		g_string_free(s,FALSE);
 	}
 
 	/* done*/
 	*(GList **)data = (gpointer)g_list_append(*(GList **)data,dbmail_imap_plist_as_string(alist));
 	
-	g_list_foreach(alist,(GFunc)g_free,NULL);
-	g_list_free(alist);
-	if (GMIME_IS_MESSAGE(part)) g_object_unref(object);
+	g_list_destroy(alist);
 
+	if (GMIME_IS_MESSAGE(part)) g_object_unref(object);
 	
 }
+
 void _structure_part_message_rfc822(GMimeObject *part, gpointer data, gboolean extension)
 {
 	char *result, *b;
@@ -1693,8 +1758,8 @@ void _structure_part_message_rfc822(GMimeObject *part, gpointer data, gboolean e
 	/* done*/
 	*(GList **)data = (gpointer)g_list_append(*(GList **)data,dbmail_imap_plist_as_string(list));
 	
-	g_list_foreach(list,(GFunc)g_free,NULL);
-	g_list_free(list);
+	g_list_destroy(list);
+
 	if (GMIME_IS_MESSAGE(part)) g_object_unref(object);
 
 }
@@ -1754,8 +1819,8 @@ void _structure_part_text(GMimeObject *part, gpointer data, gboolean extension)
 	/* done*/
 	*(GList **)data = (gpointer)g_list_append(*(GList **)data, dbmail_imap_plist_as_string(list));
 	
-	g_list_foreach(list,(GFunc)g_free,NULL);
-	g_list_free(list);
+	g_list_destroy(list);
+
 	if (GMIME_IS_MESSAGE(part)) g_object_unref(object);
 }
 
@@ -1811,8 +1876,7 @@ GList* dbmail_imap_append_alist_as_plist(GList *list, const InternetAddressList
 			}
 			g_free(s);
 			
-			g_list_foreach(t, (GFunc)g_free, NULL);
-			g_list_free(t);
+			g_list_destroy(t);
 			t = NULL;
 
 			/* Address list ending. */
@@ -1826,6 +1890,8 @@ GList* dbmail_imap_append_alist_as_plist(GList *list, const InternetAddressList
 			/* personal name */
 			if (ia->name && ia->value.addr) {
 				name = g_mime_utils_header_encode_phrase((unsigned char *)ia->name);
+				g_strdelimit(name,"\"\\",' ');
+				g_strstrip(name);
 				s = dbmail_imap_astring_as_string(name);
 				t = g_list_append_printf(t, "%s", s);
 				g_free(name);
@@ -1866,8 +1932,7 @@ GList* dbmail_imap_append_alist_as_plist(GList *list, const InternetAddressList
 			p = g_list_append_printf(p, "%s", s);
 			g_free(s);
 			
-			g_list_foreach(t, (GFunc)g_free, NULL);
-			g_list_free(t);
+			g_list_destroy(t);
 			t = NULL;
 
 			break;
@@ -1890,8 +1955,7 @@ GList* dbmail_imap_append_alist_as_plist(GList *list, const InternetAddressList
 		g_free(s);
 		g_free(st);
         
-		g_list_foreach(p, (GFunc)g_free, NULL);
-		g_list_free(p);
+		g_list_destroy(p);
 	} else {
 		list = g_list_append_printf(list, "NIL");
 	}
@@ -1933,22 +1997,27 @@ char * imap_get_structure(GMimeMessage *message, gboolean extension)
 	t = dbmail_imap_plist_collapse(s);
 	g_free(s);
 
-	g_list_foreach(structure,(GFunc)g_free,NULL);
-	g_list_free(structure);
+	g_list_destroy(structure);
 	g_object_unref(part);
 	
 	return t;
 }
 
-GList * envelope_address_part(GList *list, GMimeMessage *message, const char *header)
+static GList * envelope_address_part(GList *list, GMimeMessage *message, const char *header)
 {
 	const char *result;
 	char *t;
 	InternetAddressList *alist;
+	char *result_enc, *charset;
 	
+	charset = message_get_charset(message);
+
 	result = g_mime_message_get_header(message,header);
+	
 	if (result) {
-		t = imap_cleanup_address(result);
+		result_enc = convert_8bit_field_to_utf8(result, charset);
+		t = imap_cleanup_address(result_enc);
+		g_free(result_enc);
 		alist = internet_address_parse_string(t);
 		g_free(t);
 		list = dbmail_imap_append_alist_as_plist(list, (const InternetAddressList *)alist);
@@ -1957,9 +2026,232 @@ GList * envelope_address_part(GList *list, GMimeMessage *message, const char *he
 	} else {
 		list = g_list_append_printf(list,"NIL");
 	}
+
+	g_free(charset);
 	return list;
 }
+
+
+static void  get_msg_charset_frompart(GMimeObject *part, gpointer data)
+{
+	const char *charset=NULL;
+	if (*((char **)data)==NULL && (charset=g_mime_object_get_content_type_parameter(part,"charset"))) {
+	        *((char **)data)=g_strdup(charset);
+	}
+	return;
+}
+
+
+char * message_get_charset(GMimeMessage *message)
+{
+	GMimeObject *mime_part=NULL;
+	char *mess_charset=NULL;
+
+	if (message)
+		mime_part=g_mime_message_get_mime_part(message);
 	
+	if (mime_part) {
+		const char * charset = NULL;
+		if ((charset=g_mime_object_get_content_type_parameter(mime_part,"charset")))
+			mess_charset=g_strdup(charset);
+		g_object_unref(mime_part);
+	}
+	if (mess_charset==NULL)
+		g_mime_message_foreach_part(message,get_msg_charset_frompart,&mess_charset);
+
+	return mess_charset;
+}
+
+
+/* convert not encoded field to utf8 */
+char * convert_8bit_field_to_utf8(const char* str_in, const char *charset)
+{
+	char * subj=NULL;
+	static iconv_t default_iconv=(iconv_t)-1;
+	static int allocated_default_iconv = 0;
+	iconv_t conv_iconv;
+	
+	// Init static vars
+	if (default_iconv==(iconv_t)-1){
+		field_t val;
+		const char *default_charset=NULL;
+		GETCONFIGVALUE("DEFAULT_MSG_ENCODING", "DBMAIL", val);
+		if (val[0])
+			default_charset=val;
+		else
+			default_charset="x-unknown";// use locale charset
+		default_iconv=g_mime_iconv_open("UTF-8",default_charset);
+		if (default_iconv == (iconv_t)-1) {
+			TRACE(TRACE_DEBUG,"incorrect default encoding [%s]", default_charset);
+		}else
+			allocated_default_iconv = 1;
+	}
+	// End init
+	
+	if (str_in==NULL) {
+		return NULL;
+	}
+
+	if (g_utf8_validate((const gchar *)str_in, -1, NULL) || !g_mime_utils_text_is_8bit((unsigned char *)str_in, strlen(str_in))) {
+		// Conversion not needed
+		return g_strdup(str_in);
+	}
+
+ 	if (charset) {
+ 		// codepage not set in message header use default
+ 		TRACE(TRACE_DEBUG,"encoding 8bit use charset [%s]", charset);
+  			
+ 		if ((conv_iconv=g_mime_iconv_open("UTF-8",charset))==(iconv_t)-1) {
+			TRACE(TRACE_DEBUG,"incorrect encoding [%s] base [UTF-8]", charset);
+			subj=g_mime_iconv_strdup(default_iconv,str_in);
+		} else {
+			subj=g_mime_iconv_strdup(conv_iconv,str_in);
+			g_mime_iconv_close(conv_iconv);
+		}
+	}
+	if (subj==NULL) {
+		subj=g_mime_iconv_strdup(default_iconv,str_in);
+	}
+	    
+	if (subj==NULL) {
+		// On Error convertion,  replace all 8 bit symbol with '?'
+		subj=g_strdup(str_in);
+		char *p;
+		for(p=subj;*p;p++)
+		    if(*p & 0x80) *p='?';
+	}
+ 
+	return subj;
+}
+
+
+/* convert not encoded field to database encoding */
+char * convert_8bit_field(const char* str_in, const char *charset)
+{
+	char * subj=NULL;
+	static const char * base_charset=NULL;
+	static iconv_t base_iconv=(iconv_t)-1;
+	static iconv_t default_iconv=(iconv_t)-1;
+	iconv_t conv_iconv;
+
+	// Init static vars
+	if(base_charset==NULL){
+		field_t val;
+		GETCONFIGVALUE("ENCODING", "DBMAIL", val);
+		if(val[0]) {
+			base_charset=val;
+			TRACE(TRACE_DEBUG,"Base charset [%s]", base_charset);
+			iconv_t tmp_i=g_mime_iconv_open(base_charset,"UTF-8");
+			if(tmp_i == (iconv_t)-1) {
+				base_charset=g_mime_locale_charset();
+				TRACE(TRACE_DEBUG,"Base charset test filed set to [%s]", base_charset);
+			}
+			else
+				g_mime_iconv_close(tmp_i);
+		}
+		else
+		        base_charset=g_mime_locale_charset();
+		base_charset=g_strdup(base_charset);
+		base_iconv=g_mime_iconv_open(base_charset,base_charset);
+		if (base_iconv == (iconv_t)-1)
+			TRACE(TRACE_DEBUG,"incorrect base encoding [%s]", base_charset);
+			
+	}
+	// End init static
+	
+	if (base_iconv == (iconv_t)-1) {
+		const char *default_charset=NULL;
+		field_t val;
+		GETCONFIGVALUE("DEFAULT_MSG_ENCODING", "DBMAIL", val);
+		if (val[0])
+			default_charset=val;
+		else
+			default_charset="x-unknown";// use locale charset
+		default_iconv=g_mime_iconv_open(base_charset,default_charset);
+		if (default_iconv == (iconv_t)-1) {
+			TRACE(TRACE_DEBUG,"incorrect default encoding [%s]", default_charset);
+		}
+	}
+	if (str_in==NULL)
+		return NULL;
+	
+	if (!g_mime_utils_text_is_8bit((unsigned char *)str_in, strlen(str_in))) {
+		// No convertion required
+		return g_strdup(str_in);
+	}
+
+	if ((subj=g_mime_iconv_strdup(base_iconv,str_in))!=NULL) {
+		// Conversion already done by header decode ? May insert to database
+		return subj;
+	}
+
+ 	if (charset) {
+ 	        TRACE(TRACE_DEBUG,"encoding 8bit use charset [%s]", charset);
+  			
+ 		if ((conv_iconv=g_mime_iconv_open(base_charset,charset))==(iconv_t)-1) {
+ 			TRACE(TRACE_WARNING,"incorrect encoding [%s] base [%s]", charset,base_charset);
+ 			subj=g_mime_iconv_strdup(default_iconv,str_in);
+ 		} else {
+ 			subj=g_mime_iconv_strdup(conv_iconv,str_in);
+ 			g_mime_iconv_close(conv_iconv);
+  		}
+  	}
+    
+	if (subj==NULL) {
+		// On Error convertion,  replace all 8 bit symbol with '?'
+		subj=g_strdup(str_in);
+		char *p;
+		for(p=subj;*p;p++)
+		    if(*p & 0x80) *p='?';
+	}
+
+	return subj;
+}
+/* encode string from database encoding to mime (7-bit) */
+char * convert_8bit_db_to_mime(const char* str_in)
+{
+	char * subj=NULL;
+	static iconv_t base_iconv=(iconv_t)-1;
+	
+	if (base_iconv == (iconv_t)-1) { //Init
+		field_t val;
+		const char * base_charset=NULL;
+		GETCONFIGVALUE("ENCODING", "DBMAIL", val);
+		if(val[0]) {
+			base_charset=val;
+			TRACE(TRACE_DEBUG,"Base charset [%s]", base_charset);
+			iconv_t tmp_i=g_mime_iconv_open("UTF-8", base_charset);
+			if(tmp_i == (iconv_t)-1) {
+				base_charset=g_mime_locale_charset();
+				TRACE(TRACE_DEBUG,"Base charset test filed set to [%s]", base_charset);
+			}
+			else
+				g_mime_iconv_close(tmp_i);
+		}
+		else
+		        base_charset=g_mime_locale_charset();
+		base_charset=g_strdup(base_charset);
+		base_iconv=g_mime_iconv_open("UTF-8", base_charset);
+		if (base_iconv == (iconv_t)-1)
+			TRACE(TRACE_DEBUG,"incorrect base encoding [%s]", base_charset);
+	}
+	
+	if (str_in==NULL)
+		return NULL;
+	
+	if (!g_mime_utils_text_is_8bit((unsigned char *)str_in, strlen(str_in)))
+		return g_strdup(str_in);
+
+	if ((! g_utf8_validate((const char *)str_in,-1,NULL)) && ((subj=g_mime_iconv_strdup(base_iconv,str_in))!=NULL)) {
+ 		gchar *subj2;
+		subj2 = g_mime_utils_header_encode_text((unsigned char *)subj);
+  		g_free(subj);
+ 		return subj2;
+  	}
+
+	return g_mime_utils_header_encode_text((unsigned char *)str_in);
+}
+
 
 /* envelope access point */
 char * imap_get_envelope(GMimeMessage *message)
@@ -1967,7 +2259,7 @@ char * imap_get_envelope(GMimeMessage *message)
 	GMimeObject *part;
 	GList *list = NULL;
 	char *result;
-	char *s, *t;
+	char *s = NULL, *t = NULL;
 
 	if (! GMIME_IS_MESSAGE(message))
 		return NULL;
@@ -1987,8 +2279,16 @@ char * imap_get_envelope(GMimeMessage *message)
 	
 	/* subject */
 	result = (char *)g_mime_message_get_header(message,"Subject");
+
 	if (result) {
-		t = dbmail_imap_astring_as_string(result);
+		char *charset = message_get_charset(message);
+		char * subj = convert_8bit_field_to_utf8(result, charset);
+		g_free(charset);
+		s = g_mime_utils_header_encode_text((unsigned char *)subj);
+		TRACE(TRACE_DEBUG,"encoding 8bit subject [%s] -> [%s]", subj, s);
+		t = dbmail_imap_astring_as_string(s);
+		g_free(s);
+		g_free(subj);
 		list = g_list_append_printf(list,"%s", t);
 		g_free(t);
 	} else {
@@ -2032,8 +2332,7 @@ char * imap_get_envelope(GMimeMessage *message)
 
 	s = dbmail_imap_plist_as_string(list);
 
-	g_list_foreach(list,(GFunc)g_free,NULL);
-	g_list_free(list);
+	g_list_destroy(list);
 	
 	return s;
 }
@@ -2070,37 +2369,60 @@ GMimeObject * imap_get_partspec(const GMimeObject *message, const char *partspec
 	char *part;
 	guint index;
 	guint i;
+
+	assert(message);
+	assert(partspec);
 	
 	GString *t = g_string_new(partspec);
 	GList *specs = g_string_split(t,".");
 	g_string_free(t,TRUE);
 	
 	object = GMIME_OBJECT(message);
+	if (!object) {
+		TRACE(TRACE_INFO, "message is not an object");
+		return NULL;
+	}
+		
 	for (i=0; i< g_list_length(specs); i++) {
 		part = g_list_nth_data(specs,i);
 		if (! (index = strtol((const char *)part, NULL, 0))) 
 			break;
 		
 		if (GMIME_IS_MESSAGE(object))
-			object=GMIME_OBJECT(GMIME_MESSAGE(object)->mime_part);
+			object = GMIME_OBJECT(GMIME_MESSAGE(object)->mime_part);
 		
 		type = (GMimeContentType *)g_mime_object_get_content_type(object);
 
 		if (g_mime_content_type_is_type(type,"multipart","*")) {
-			object=g_mime_multipart_get_part((GMimeMultipart *)object, (int)index-1);
+			object = g_mime_multipart_get_part((GMimeMultipart *)object, (int)index-1);
+			if (!object) {
+				TRACE(TRACE_INFO, "object part [%d] is null", (int)index-1);
+				return NULL;
+			}
+			if (! GMIME_IS_OBJECT(object)) {
+				TRACE(TRACE_INFO, "object part [%d] is not an object", (int)index-1);
+				return NULL;
+			}
+
 			type = (GMimeContentType *)g_mime_object_get_content_type(object);
-			assert(object);
 		}
 
 		// for message/rfc822 parts we want the contained message, 
 		// not the mime-part as such
 
 		if (g_mime_content_type_is_type(type,"message","rfc822")) {
-			object=GMIME_OBJECT(GMIME_MESSAGE_PART(object)->message);
-			assert(object);
+			object = GMIME_OBJECT(GMIME_MESSAGE_PART(object)->message);
+			if (!object) {
+				TRACE(TRACE_INFO, "rfc822 part is null");
+				return NULL;
+			}
+			if (! GMIME_IS_OBJECT(object)) {
+				TRACE(TRACE_INFO, "rfc822 part is not an object");
+				return NULL;
+			}
 		}
-	
 	}
+
 	return object;
 }
 
@@ -2112,9 +2434,17 @@ char * imap_cleanup_address(const char *a)
 	char prev,next=0;
 	unsigned incode=0, inquote=0;
 	size_t i, l;
-	GString *s = g_string_new("");
+	GString *s;
+
+	if (!a || !a[0])
+		return g_strdup("");
 	
+	s = g_string_new("");
 	t = g_strdup(a);
+
+	// un-fold and collapse tabs and spaces
+	g_strdelimit(t,"\n",' ');
+	dm_pack_spaces(t);
 	inptr = t;
 	inptr = g_strstrip(inptr);
 	prev = *inptr;
@@ -2130,8 +2460,7 @@ char * imap_cleanup_address(const char *a)
 	}
 
 	l = strlen(inptr);
-
-	for (i=0; i<l-1; i++) {
+	for (i = 0; i < l - 1; i++) {
 
 		next=inptr[i+1];
 
@@ -2196,4 +2525,19 @@ char * imap_cleanup_address(const char *a)
 	return r;
 }
 
+char * imap_flags_as_string(msginfo_t *msginfo)
+{
+	GList *sublist = NULL;
+	int j;
+	char *s;
+
+	for (j = 0; j < IMAP_NFLAGS; j++) {
+		if (msginfo->flags[j])
+			sublist = g_list_append(sublist,g_strdup((gchar *)imap_flag_desc_escaped[j]));
+	}
+	s = dbmail_imap_plist_as_string(sublist);
+	g_list_destroy(sublist);
+	return s;
+}
+
 
