@@ -18,24 +18,17 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/*
- * $Id: authldap.c 2347 2006-11-01 21:35:52Z paul $
- * * User authentication functions for LDAP.
- */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+/* User authentication functions for LDAP. */
 
 #include "dbmail.h"
+#define THIS_MODULE "auth"
 
 #define AUTH_QUERY_SIZE 1024
 #define LDAP_RES_SIZE 1024
-#define THIS_MODULE "auth"
 
 extern char *configFile;
 
 static LDAP *_ldap_conn = NULL;
-LDAPMod **_ldap_mod;
 LDAPMessage *_ldap_res;
 LDAPMessage *_ldap_msg;
 int _ldap_err;
@@ -46,7 +39,7 @@ char **_ldap_attrs = NULL;
 char _ldap_query[AUTH_QUERY_SIZE];
 
 typedef struct _ldap_cfg {
-	field_t bind_dn, bind_pw, base_dn, port, version, scope, hostname;
+	field_t bind_dn, bind_pw, base_dn, port, uri, version, scope, hostname;
 	field_t user_objectclass, forw_objectclass;
 	field_t cn_string;
 	field_t field_uid, field_cid, min_cid, max_cid, field_nid, min_nid, max_nid;
@@ -76,6 +69,7 @@ static void __auth_get_config(void)
 	GETCONFIGVALUE("BIND_PW",		"LDAP", _ldap_cfg.bind_pw);
 	GETCONFIGVALUE("BASE_DN",		"LDAP", _ldap_cfg.base_dn);
 	GETCONFIGVALUE("PORT",			"LDAP", _ldap_cfg.port);
+	GETCONFIGVALUE("URI",			"LDAP", _ldap_cfg.uri);
 	GETCONFIGVALUE("VERSION",		"LDAP", _ldap_cfg.version);
 	GETCONFIGVALUE("HOSTNAME",		"LDAP", _ldap_cfg.hostname);
 	GETCONFIGVALUE("USER_OBJECTCLASS",	"LDAP", _ldap_cfg.user_objectclass);
@@ -152,30 +146,67 @@ static int auth_ldap_bind(void)
  */
 int auth_connect(void)
 {
-	int version;
+	int version = 0;
+#ifdef HAVE_LDAP_INITIALIZE
+	int ret;
+	char *uri;
+#endif
 
 	if (_ldap_conn != NULL) 
 		return 0;
 	
 	__auth_get_config();
-	
-	TRACE(TRACE_DEBUG, "connecting to ldap server on [%s] : [%d] version [%d]",
-		_ldap_cfg.hostname, _ldap_cfg.port_int, _ldap_cfg.version_int);
-	
-	_ldap_conn = ldap_init(
-			_ldap_cfg.hostname, 
-			_ldap_cfg.port_int);
 
 	switch (_ldap_cfg.version_int) {
-	case 2:
-		version = LDAP_VERSION2;
-		break;
 	case 3:
 		version = LDAP_VERSION3;
+		if (strlen(_ldap_cfg.uri)) {
+#ifdef HAVE_LDAP_INITIALIZE
+			TRACE(TRACE_DEBUG, 
+				"connecting to ldap server on [%s] version [%d]", 
+				_ldap_cfg.uri, _ldap_cfg.version_int);
+			if ((ret = ldap_initialize(&_ldap_conn, _ldap_cfg.uri) 
+				== LDAP_SUCCESS)) 
+				break;
+			else 
+				TRACE(TRACE_WARNING, "ldap_initialize() returned %d", ret);
+#else
+			TRACE(TRACE_WARNING, "LDAP library doesn't support ldap_initialize()."
+				" You cannot use the URI directive to specify the DSN.");
+#endif
+		}
+
+#ifdef HAVE_LDAP_INITIALIZE
+		uri = g_strdup_printf("ldap://%s:%d", _ldap_cfg.hostname, _ldap_cfg.port_int);
+
+		TRACE(TRACE_DEBUG, 
+			"connecting to ldap server on [%s] version [%d]", 
+			uri, _ldap_cfg.version_int);
+		if ((ret = ldap_initialize(&_ldap_conn, uri)) != LDAP_SUCCESS) 
+			TRACE(TRACE_FATAL, "ldap_initialize() returned [%d]", ret);
+
+		g_free(uri);
+#else
+		TRACE(TRACE_DEBUG, 
+			"connecting to ldap server on [%s] : [%d] version [%d]",
+			_ldap_cfg.hostname, _ldap_cfg.port_int, _ldap_cfg.version_int);
+		_ldap_conn = ldap_init(_ldap_cfg.hostname, _ldap_cfg.port_int);
+#endif
 		break;
+	case 2:
+		version = LDAP_VERSION2;
+		/* fall through... */
 	default:
-		TRACE(TRACE_ERROR, "Unsupported LDAP version requested [%d]. Defaulting to LDAP version 3.", _ldap_cfg.version_int);
-		version = LDAP_VERSION3;
+		if (!version) {
+			TRACE(TRACE_WARNING, "Unsupported LDAP version [%d] requested."
+				" Default to LDAP version 3.", _ldap_cfg.version_int);
+			version = LDAP_VERSION3;
+		}
+
+		TRACE(TRACE_DEBUG, 
+			"connecting to ldap server on [%s] : [%d] version [%d]",
+			_ldap_cfg.hostname, _ldap_cfg.port_int, _ldap_cfg.version_int);
+		_ldap_conn = ldap_init(_ldap_cfg.hostname, _ldap_cfg.port_int);
 		break;
 	}
 
@@ -679,7 +710,7 @@ int auth_check_userid(u64_t user_idnr)
 		TRACE(TRACE_DEBUG, "didn't find user_idnr [%llu]", user_idnr);
 	}
 
-	dm_free(returnid);
+	g_free(returnid);
 
 	return ret;
 }
@@ -889,16 +920,13 @@ int auth_adduser(const char *username, const char *password,
 		 const char *enctype UNUSED, u64_t clientid, 
 		 u64_t maxmail, u64_t * user_idnr)
 {
-	int i, j, result;
-	/*int ret; unused variable */
-	int NUM_MODS = 9;
+	int i = 0, result;
 	GString *nid = g_string_new("");
 	GString *cid = g_string_new("");
 	GString *maxm = g_string_new("");
 	
 	u64_t newidnr = dm_ldap_get_freeid(_ldap_cfg.field_nid);
 
-	
 	g_string_printf(nid,"%llu", newidnr);
 	g_string_printf(cid,"%llu",clientid);
 	g_string_printf(maxm,"%llu",maxmail);
@@ -912,80 +940,64 @@ int auth_adduser(const char *username, const char *password,
 	
 	field_t mail_type = "mail";
 	field_t obj_type = "objectClass";
-	
+
 	GString *t=g_string_new("");
 	
 	assert(user_idnr != NULL);
 	*user_idnr = 0;
 
-	/* Construct the array of LDAPMod structures representing the attributes */ 
-	if (! (_ldap_mod = (LDAPMod **) dm_malloc((NUM_MODS + 1) * sizeof(LDAPMod *)))) {
-		TRACE(TRACE_ERROR, "Cannot allocate memory for mods array");
-		return -1;
-	}
-
-	for (i = 0; i < NUM_MODS; i++) {
-		if (! (_ldap_mod[i] = (LDAPMod *) dm_malloc(sizeof(LDAPMod)))) {
-			TRACE(TRACE_ERROR, "Cannot allocate memory for mods element %d", i);
-			for (j = 0; j < (i - 1); j++)
-				dm_free(_ldap_mod[j]);
-			dm_free(_ldap_mod);
-			return -1;
-		}
-	}
-
 	g_string_printf(t,"%s=%s,%s", _ldap_cfg.cn_string, username, _ldap_cfg.base_dn);
 	_ldap_dn=g_strdup(t->str);
 	g_string_free(t,TRUE);
 	
 	TRACE(TRACE_DEBUG, "Adding user with DN of [%s]", _ldap_dn);
 	
-	i = 0;
-	_ldap_mod[i]->mod_op = LDAP_MOD_ADD;
-	_ldap_mod[i]->mod_type = obj_type;
-	_ldap_mod[i]->mod_values = obj_values;
+	LDAPMod *mods[10], mod_obj_type, mod_field_passwd, mod_mail_type,
+	mod_field_uid, mod_field_cid, mod_field_maxmail, mod_field_nid;
+
+	
+	mod_obj_type.mod_op = LDAP_MOD_ADD;
+	mod_obj_type.mod_type = obj_type;
+	mod_obj_type.mod_values = obj_values;
+	mods[i++] = &mod_obj_type;
 	
 	if (strlen(_ldap_cfg.field_passwd) > 0) {
-		i++;
-		_ldap_mod[i]->mod_op = LDAP_MOD_ADD;
-		_ldap_mod[i]->mod_type = _ldap_cfg.field_passwd;
-		_ldap_mod[i]->mod_values = pw_values;
+		mod_field_passwd.mod_op = LDAP_MOD_ADD;
+		mod_field_passwd.mod_type = _ldap_cfg.field_passwd;
+		mod_field_passwd.mod_values = pw_values;
+		mods[i++] = &mod_field_passwd;
 	}
 
-	i++;
-	_ldap_mod[i]->mod_op = LDAP_MOD_ADD;
-	_ldap_mod[i]->mod_type = mail_type;
-	_ldap_mod[i]->mod_values = uid_values;
+	mod_mail_type.mod_op = LDAP_MOD_ADD;
+	mod_mail_type.mod_type = mail_type;
+	mod_mail_type.mod_values = uid_values;
+	mods[i++] = &mod_mail_type;
 
-	i++;
-	_ldap_mod[i]->mod_op = LDAP_MOD_ADD;
-	_ldap_mod[i]->mod_type = _ldap_cfg.field_uid;
-	_ldap_mod[i]->mod_values = uid_values;
+	mod_field_uid.mod_op = LDAP_MOD_ADD;
+	mod_field_uid.mod_type = _ldap_cfg.field_uid;
+	mod_field_uid.mod_values = uid_values;
+	mods[i++] = &mod_field_uid;
 	
-	i++;
-	_ldap_mod[i]->mod_op = LDAP_MOD_ADD;
-	_ldap_mod[i]->mod_type = _ldap_cfg.field_cid;
-	_ldap_mod[i]->mod_values = cid_values;
+	mod_field_cid.mod_op = LDAP_MOD_ADD;
+	mod_field_cid.mod_type = _ldap_cfg.field_cid;
+	mod_field_cid.mod_values = cid_values;
+	mods[i++] = &mod_field_cid;
 
-	i++;
-	_ldap_mod[i]->mod_op = LDAP_MOD_ADD;
-	_ldap_mod[i]->mod_type = _ldap_cfg.field_maxmail;
-	_ldap_mod[i]->mod_values = max_values;
+	mod_field_maxmail.mod_op = LDAP_MOD_ADD;
+	mod_field_maxmail.mod_type = _ldap_cfg.field_maxmail;
+	mod_field_maxmail.mod_values = max_values;
+	mods[i++] = &mod_field_maxmail;
 
-	i++;
-	_ldap_mod[i]->mod_op = LDAP_MOD_ADD;
-	_ldap_mod[i]->mod_type = _ldap_cfg.field_nid;
-	_ldap_mod[i]->mod_values = nid_values;
+	mod_field_nid.mod_op = LDAP_MOD_ADD;
+	mod_field_nid.mod_type = _ldap_cfg.field_nid;
+	mod_field_nid.mod_values = nid_values;
+	mods[i++] = &mod_field_nid;
 
-	i++;
-	_ldap_mod[i] = NULL;
+	mods[i++] = NULL;
 
-	_ldap_err = ldap_add_s(_ldap_conn, _ldap_dn, _ldap_mod);
+	_ldap_err = ldap_add_s(_ldap_conn, _ldap_dn, mods);
 
 	g_strfreev(obj_values);
-	for (i = 0; i < NUM_MODS; i++)
-		dm_free(_ldap_mod[i]);
-	dm_free(_ldap_mod);
 	ldap_memfree(_ldap_dn);
 
 	if (_ldap_err) {
@@ -1194,13 +1206,13 @@ int auth_validate(clientinfo_t *ci, char *username, char *password, u64_t * user
 			return DM_EQUERY;
 	}
 
-	if (auth_user_exists(real_username, user_idnr) == -1) {
-		return -1;
+	if (auth_user_exists(real_username, user_idnr) != 1) {
+		return 0;
 	}
 	
 	if (! (ldap_dn = dm_ldap_user_getdn(*user_idnr))) {
 		TRACE(TRACE_ERROR,"unable to determine DN for user");
-		return -1;
+		return 0;
 	}
 
 	/* now, try to rebind as the given DN using the supplied password */
@@ -1440,7 +1452,7 @@ static int forward_exists(const char *alias, const char *deliver_to)
 		
 	
 	g_free(objectfilter);
-	dm_free(dn);
+	g_free(dn);
 	g_string_free(t,TRUE);
 	g_list_foreach(l,(GFunc)g_free,NULL);
 	
