@@ -1,4 +1,4 @@
-/* $Id: sortsieve.c 1912 2005-11-19 02:29:41Z aaron $
+/* 
 
  Copyright (C) 1999-2004 Aaron Stone aaron at serendipity dot cx
 
@@ -102,18 +102,23 @@ We need to make sure to respect the implementation requirements.
 int sort_vacation(sieve2_context_t *s, void *my)
 {
 	struct sort_context *m = (struct sort_context *)my;
-	int days = 1, mime = 0;
 	const char *message, *subject, *fromaddr, *handle;
 	const char *rc_to, *rc_from, *rc_handle;
 	char *md5_handle = NULL;
+	int days, mime;
 
-	days = sieve2_getvalue_int(s, "days"); // days: min 1, max 30, default 7.
+	days = sieve2_getvalue_int(s, "days");
 	mime = sieve2_getvalue_int(s, "mime"); // mime: 1 if message is mime coded. FIXME.
 	message = sieve2_getvalue_string(s, "message");
 	subject = sieve2_getvalue_string(s, "subject");
 	fromaddr = sieve2_getvalue_string(s, "fromaddr"); // From: specified by the script.
 	handle = sieve2_getvalue_string(s, "handle");
 
+	/* Default to a week, upper limit of a month.
+	 * This is our only loop prevention mechanism! The value must be
+	 * greater than 0, else the replycache code will always indicate
+	 * that we haven't seen anything since 0 days ago... */
+	if (days == 0) days = 7;
 	if (days < 1) days = 1;
 	if (days > 30) days = 30;
 
@@ -149,7 +154,7 @@ int sort_vacation(sieve2_context_t *s, void *my)
 	}
 
 	if (md5_handle)
-		dm_free(md5_handle);
+		g_free(md5_handle);
 
 	m->result->cancelkeep = 0;
 	return SIEVE2_OK;
@@ -529,7 +534,7 @@ static int sort_teardown(sieve2_context_t **s2c,
 	dm_list_free(&sort_context->freelist.start);
 
 	if (sort_context) {
-		dm_free(sort_context);
+		g_free(sort_context);
 	}
 
 	res = sieve2_free(&sieve2_context);
@@ -603,7 +608,7 @@ static int sort_startup(sieve2_context_t **s2c,
 		}
 	}
 
-	sort_context = dm_malloc(sizeof(struct sort_context));
+	sort_context = g_new0(struct sort_context, 1);
 	if (!sort_context) {
 		sort_teardown(&sieve2_context, &sort_context);
 		return DM_EGENERAL;
@@ -651,7 +656,7 @@ const char * sort_listextensions(void)
 
 	/* So we'll make our own copy. */
 	if (extensions)
-		extensions = dm_strdup(extensions);
+		extensions = g_strdup(extensions);
 
 	/* If this fails, then we don't care about the
 	 * memory leak, because the program has to bomb out.
@@ -700,7 +705,7 @@ sort_result_t *sort_validate(u64_t user_idnr, char *scriptname)
 
 freesieve:
 	if (sort_context->s_buf)
-		dm_free(sort_context->s_buf);
+		g_free(sort_context->s_buf);
 
 	if (exitnull)
 		result = NULL;
@@ -773,9 +778,9 @@ sort_result_t *sort_process(u64_t user_idnr, struct DbmailMessage *message)
 
 freesieve:
 	if (sort_context->s_buf)
-		dm_free(sort_context->s_buf);
+		g_free(sort_context->s_buf);
 	if (sort_context->script)
-		dm_free(sort_context->script);
+		g_free(sort_context->script);
 
 	if (exitnull)
 		result = NULL;
@@ -796,7 +801,7 @@ void sort_free_result(sort_result_t *result)
 		g_string_free(result->errormsg, TRUE);
 	if (result->rejectmsg != NULL) 
 		g_string_free(result->rejectmsg, TRUE);
-	dm_free(result);
+	g_free(result);
 }
 
 int sort_get_cancelkeep(sort_result_t *result)
