@@ -18,7 +18,7 @@
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* $Id: pop3.c 2356 2006-11-05 16:43:48Z aaron $
+/* 
  *
  * implementation for pop3 commands according to RFC 1081 */
 
@@ -97,7 +97,7 @@ int pop3_handle_connection(clientinfo_t * ci)
 
 	if (ci->tx) {
 		/* sending greeting */
-		fprintf(ci->tx, "+OK DBMAIL pop3 server ready to rock %s\r\n",
+		ci_write(ci->tx, "+OK DBMAIL pop3 server ready to rock %s\r\n",
 			session.apop_stamp);
 		fflush(ci->tx);
 	} else {
@@ -132,8 +132,8 @@ int pop3_handle_connection(clientinfo_t * ci)
 				
 				if (alarm_occured) {
 					alarm_occured = 0;
-					client_close();
-					done = -1;
+					done = -2;
+					session.SessionResult = 1;
 					break;
 				}
 
@@ -145,7 +145,9 @@ int pop3_handle_connection(clientinfo_t * ci)
 			}
 		}
 
-		if (feof(ci->rx) || ferror(ci->rx))
+		if (done < 0) {
+			break;
+		} else if (feof(ci->rx) || ferror(ci->rx))
 			done = -1;
 		else {
 			/* reset function handle timeout */
@@ -176,11 +178,17 @@ int pop3_handle_connection(clientinfo_t * ci)
 					session.virtual_totalsize);
 
 			/* if everything went well, write down everything and do a cleanup */
-			db_update_pop(&session);
+			if (db_update_pop(&session) == DM_SUCCESS)
+				ci_write(ci->tx, "+OK see ya later\r\n");
+			else
+				ci_write(ci->tx, "-ERR some deleted messages not removed\r\n");
+
+			fflush(ci->tx);
 			break;
 
 		case 1:
-			TRACE(TRACE_ERROR, "EOF from client, connection terminated");
+			ci_write(ci->tx, "-ERR I'm leaving, you're too slow\r\n");
+			TRACE(TRACE_ERROR, "client timed out, connection closed");
 			break;
 
 		case 2:
@@ -194,6 +202,12 @@ int pop3_handle_connection(clientinfo_t * ci)
 			TRACE(TRACE_ERROR, "storage layer failure");
 			break;
 		}
+	} else if (done==0) { // QUIT issued before AUTH phase completed
+		ci_write(ci->tx, "+OK see ya later\r\n");
+		fflush(ci->tx);
+	} else if (done==-2) {
+		ci_write(ci->tx, "-ERR I'm leaving, you're too slow\r\n");
+		TRACE(TRACE_ERROR, "client timed out before AUTH, connection closed");
 	} else {
 		TRACE(TRACE_ERROR, "error, incomplete session");
 	}
@@ -227,7 +241,7 @@ int pop3_error(PopSession_t * session, void *stream,
 	if (session->error_count >= MAX_ERRORS) {
 		TRACE(TRACE_MESSAGE, "too many errors (MAX_ERRORS is %d)", 
 				MAX_ERRORS);
-		fprintf((FILE *) stream, "-ERR loser, go play somewhere else\r\n");
+		ci_write((FILE *) stream, "-ERR loser, go play somewhere else\r\n");
 		session->SessionResult = 2;	/* possible flood */
 		return -3;
 	} else {
@@ -329,7 +343,9 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 	switch (cmdtype) {
 		
 	case POP3_QUIT:
-		fprintf((FILE *) stream, "+OK see ya later\r\n");
+		/* We return 0 here, and then pop3_handle_connection cleans up
+		 * the connection, commits all changes, and sends the final
+		 * "OK" message indicating that QUIT has completed. */
 		return 0;
 		
 	case POP3_USER:
@@ -351,7 +367,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			strncpy(session->username, value, strlen(value) + 1);
 		}
 
-		fprintf((FILE *) stream, "+OK Password required for %s\r\n", session->username);
+		ci_write((FILE *) stream, "+OK Password required for %s\r\n", session->username);
 		return 1;
 
 	case POP3_PASS:
@@ -377,7 +393,8 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			session->SessionResult = 3;
 			return -1;
 		case 0:
-			TRACE(TRACE_ERROR, "user [%s] tried to login with wrong password", session->username);
+			TRACE(TRACE_ERROR, "user [%s] coming from [%s] tried to login with wrong password", 
+				session->username, ci->ip_src);
 
 			g_free(session->username);
 			session->username = NULL;
@@ -399,9 +416,11 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			if (pop_before_smtp)
 				db_log_ip(client_ip);
 
+			child_reg_connected_user(session->username);
+
 			result = db_createsession(result, session);
 			if (result == 1) {
-				fprintf((FILE *) stream, "+OK %s has %llu messages (%llu octets)\r\n", 
+				ci_write((FILE *) stream, "+OK %s has %llu messages (%llu octets)\r\n", 
 						session->username, 
 						session->virtual_totalmessages, 
 						session->virtual_totalsize);
@@ -426,7 +445,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			while (tmpelement != NULL) {
 				msg = (struct message *)tmpelement->data;
 				if (msg->messageid == strtoull(value,NULL, 10) && msg->virtual_messagestatus < MESSAGE_STATUS_DELETE) {
-					fprintf((FILE *) stream, "+OK %llu %llu\r\n", msg->messageid,msg->msize);
+					ci_write((FILE *) stream, "+OK %llu %llu\r\n", msg->messageid,msg->msize);
 					found = 1;
 				}
 				tmpelement = tmpelement->nextnode;
@@ -438,7 +457,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 		}
 
 		/* just drop the list */
-		fprintf((FILE *) stream, "+OK %llu messages (%llu octets)\r\n", 
+		ci_write((FILE *) stream, "+OK %llu messages (%llu octets)\r\n", 
 				session->virtual_totalmessages, 
 				session->virtual_totalsize);
 
@@ -447,18 +466,18 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			while (tmpelement != NULL) {
 				msg = (struct message *)tmpelement->data;
 				if (msg->virtual_messagestatus < MESSAGE_STATUS_DELETE)
-					fprintf((FILE *) stream, "%llu %llu\r\n", msg->messageid,msg->msize);
+					ci_write((FILE *) stream, "%llu %llu\r\n", msg->messageid,msg->msize);
 				tmpelement = tmpelement->nextnode;
 			}
 		}
-		fprintf((FILE *) stream, ".\r\n");
+		ci_write((FILE *) stream, ".\r\n");
 		return 1;
 
 	case POP3_STAT:
 		if (session->state != POP3_TRANSACTION_STATE)
 			return pop3_error(session, stream, "-ERR wrong command mode\r\n");
 
-		fprintf((FILE *) stream, "+OK %llu %llu\r\n", 
+		ci_write((FILE *) stream, "+OK %llu %llu\r\n", 
 				session->virtual_totalmessages, 
 				session->virtual_totalsize);
 
@@ -479,7 +498,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			msg = (struct message *) tmpelement->data;
 			if (msg->messageid == strtoull(value, NULL, 10) && msg->virtual_messagestatus < MESSAGE_STATUS_DELETE) {	/* message is not deleted */
 				msg->virtual_messagestatus = MESSAGE_STATUS_SEEN;
-				fprintf((FILE *) stream, "+OK %llu octets\r\n", msg->msize);
+				ci_write((FILE *) stream, "+OK %llu octets\r\n", msg->msize);
 				return db_send_message_lines((void *) stream, msg->realmessageid, -2, 0);
 			}
 			tmpelement = tmpelement->nextnode;
@@ -500,7 +519,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 				session->virtual_totalsize -= msg->msize;
 				session->virtual_totalmessages -= 1;
 
-				fprintf((FILE *) stream, "+OK message %llu deleted\r\n", msg->messageid);
+				ci_write((FILE *) stream, "+OK message %llu deleted\r\n", msg->messageid);
 				return 1;
 			}
 			tmpelement = tmpelement->nextnode;
@@ -522,7 +541,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			tmpelement = tmpelement->nextnode;
 		}
 
-		fprintf((FILE *) stream, "+OK %llu messages (%llu octets)\r\n", session->virtual_totalmessages, session->virtual_totalsize);
+		ci_write((FILE *) stream, "+OK %llu messages (%llu octets)\r\n", session->virtual_totalmessages, session->virtual_totalsize);
 
 		return 1;
 
@@ -536,14 +555,14 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			msg = (struct message *) tmpelement->data;
 			if (msg->virtual_messagestatus == MESSAGE_STATUS_NEW) {
 				/* we need the last message that has been accessed */
-				fprintf((FILE *) stream, "+OK %llu\r\n", msg->messageid - 1);
+				ci_write((FILE *) stream, "+OK %llu\r\n", msg->messageid - 1);
 				return 1;
 			}
 			tmpelement = tmpelement->nextnode;
 		}
 
 		/* all old messages */
-		fprintf((FILE *) stream, "+OK %llu\r\n", session->virtual_totalmessages);
+		ci_write((FILE *) stream, "+OK %llu\r\n", session->virtual_totalmessages);
 
 		return 1;
 
@@ -551,7 +570,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 		if (session->state != POP3_TRANSACTION_STATE)
 			return pop3_error(session, stream, "-ERR wrong command mode\r\n");
 
-		fprintf((FILE *) stream, "+OK\r\n");
+		ci_write((FILE *) stream, "+OK\r\n");
 		return 1;
 
 	case POP3_UIDL:
@@ -565,7 +584,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			while (tmpelement != NULL) {
 				msg = (struct message *)tmpelement->data;
 				if (msg->messageid == strtoull(value,NULL, 10) && msg->virtual_messagestatus < MESSAGE_STATUS_DELETE) {
-					fprintf((FILE *) stream, "+OK %llu %s\r\n", msg->messageid,msg->uidl);
+					ci_write((FILE *) stream, "+OK %llu %s\r\n", msg->messageid,msg->uidl);
 					found = 1;
 				}
 				tmpelement = tmpelement->nextnode;
@@ -577,20 +596,20 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 		}
 
 		/* just drop the list */
-		fprintf((FILE *) stream, "+OK Some very unique numbers for you\r\n");
+		ci_write((FILE *) stream, "+OK Some very unique numbers for you\r\n");
 
 		if (session->virtual_totalmessages > 0) {
 			/* traversing list */
 			while (tmpelement != NULL) {
 				msg = (struct message *)tmpelement->data; 
 				if (msg->virtual_messagestatus < MESSAGE_STATUS_DELETE)
-					fprintf((FILE *) stream, "%llu %s\r\n", msg->messageid, msg->uidl);
+					ci_write((FILE *) stream, "%llu %s\r\n", msg->messageid, msg->uidl);
 
 				tmpelement = tmpelement->nextnode;
 			}
 		}
 
-		fprintf((FILE *) stream, ".\r\n");
+		ci_write((FILE *) stream, ".\r\n");
 
 		return 1;
 
@@ -675,9 +694,11 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 			if (pop_before_smtp)
 				db_log_ip(client_ip);
 
+			child_reg_connected_user(session->username);
+
 			result = db_createsession(result, session);
 			if (result == 1) {
-				fprintf((FILE *) stream, "+OK %s has %llu messages (%llu octets)\r\n", 
+				ci_write((FILE *) stream, "+OK %s has %llu messages (%llu octets)\r\n", 
 						session->username, 
 						session->virtual_totalmessages, 
 						session->virtual_totalsize);
@@ -737,7 +758,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 		while (tmpelement != NULL) {
 			msg = (struct message *) tmpelement->data;
 			if (msg->messageid == top_messageid && msg->virtual_messagestatus < MESSAGE_STATUS_DELETE) {	/* message is not deleted */
-				fprintf((FILE *) stream, "+OK %llu lines of message %llu\r\n", top_lines, top_messageid);
+				ci_write((FILE *) stream, "+OK %llu lines of message %llu\r\n", top_lines, top_messageid);
 				return db_send_message_lines(stream, msg->realmessageid, top_lines, 0);
 			}
 			tmpelement = tmpelement->nextnode;
@@ -745,7 +766,7 @@ int pop3(clientinfo_t *ci, char *buffer, PopSession_t * session)
 		return pop3_error(session, stream, "-ERR no such message\r\n");
 
 	case POP3_CAPA:
-		fprintf((FILE *) stream, "+OK, Capability list follows\r\nTOP\r\nUSER\r\nUIDL\r\n.\r\n");
+		ci_write((FILE *) stream, "+OK Capability list follows\r\nTOP\r\nUSER\r\nUIDL\r\n.\r\n");
 		return 1;
 
 	default:
