@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-# $Id: testimap.py 2371 2006-11-15 15:55:46Z paul $
+# 
 
 # For a protocol trace set to 4
 DEBUG = 0
@@ -28,7 +28,7 @@ TYPE = 'stream'
 #TYPE = 'network'
 
 
-import unittest, imaplib, re
+import unittest, imaplib, re, commands
 import sys, traceback, getopt, string
 from email.MIMEText import MIMEText
 from email.MIMEMultipart import MIMEMultipart
@@ -42,7 +42,7 @@ HOST,PORT = "localhost", 143
 # for stdin/stdout testing
 DAEMONBIN = "./dbmail-imapd -n -f /etc/dbmail/dbmail-test.conf"
 # with valgrind
-#DAEMONBIN = "valgrind --suppressions=./contrib/dbmail.supp --leak-check=full %s" % DAEMONBIN
+#DAEMONBIN = "CK_FORK=no G_SLICE=always-malloc valgrind --suppressions=./contrib/dbmail.supp --leak-check=full %s" % DAEMONBIN
 
 
 TESTMSG={}
@@ -81,6 +81,17 @@ def getsock():
 def strip_crlf(s):
     return string.replace(s,'\r','')
 
+def getFreshbox(name):
+    assert(name)
+    cmd='./contrib/mailbox2dbmail/mailbox2dbmail ' \
+        '-u testuser1 -t mbox -m test-scripts/testbox ' \
+        '-b "%s" -p ./dbmail-smtp -f /etc/dbmail/dbmail-test.conf' % name
+    print "loading new testbox [%s]. please wait..." % name
+    s,o=commands.getstatusoutput(cmd)
+    if s:
+        raise "error", o
+
+
 class testImapServer(unittest.TestCase):
 
     def setUp(self,username="testuser1",password="test"):
@@ -102,8 +113,8 @@ class testImapServer(unittest.TestCase):
         # test flags
         self.o.create('testappend')
         self.o.append('testappend','\Flagged',"\" 3-Mar-2006 07:15:00 +0200 \"",str(TESTMSG['strict822']))
-        self.o.select('testappend')
-        id=self.o.recent()[1][0]
+        result = self.o.select('testappend')
+        id = result[1][0]
         
         result = self.o.fetch(id,"(UID BODY[])")
         self.assertEquals(result[0],'OK')
@@ -146,10 +157,12 @@ class testImapServer(unittest.TestCase):
         """
         self.o.create('testcopy1')
         self.o.create('testcopy2')
-        self.o.append('testcopy1',"","",str(TESTMSG['strict822']))
+        for i in range(1,20):
+            self.o.append('testcopy1',"","",str(TESTMSG['strict822']))
         self.o.select('testcopy1')
         id = self.o.recent()[1][0]
         self.assertEquals(self.o.copy(id,'testcopy2'),('OK', ['COPY completed']))
+        self.assertEquals(self.o.copy('1:*','testcopy2'),('OK', ['COPY completed']))
 
     def testCreate(self):
         """ 
@@ -185,8 +198,29 @@ class testImapServer(unittest.TestCase):
             an `EXPUNGE' response for each deleted message. Returned data
             contains a list of `EXPUNGE' message numbers in order received.
         """
-        self.o.select('INBOX')
-        self.assertEquals(self.o.expunge(),('OK', [None]))
+
+        getFreshbox('testexpungebox')
+
+        p = getsock()
+        p.debug = DEBUG
+        p.login('testuser1','test'),('OK',['LOGIN completed'])
+
+        self.o.select('testexpungebox')
+        
+        p.select('testexpungebox')
+        
+        self.o.store('5:*', '+FLAGS', '\Deleted')
+        self.o.debug = 4
+        msnlist = self.o.expunge()[1];
+        self.assertEquals(msnlist,['11', '10', '9', '8', '7', '6', '5'])
+        self.assertEquals(self.o.fetch('4','(BODYSTRUCTURE FLAGS)')[0],'OK')
+        
+        p.debug = 4
+        print p.noop()
+
+        self.o.debug = 0
+        p.debug = 0
+
 
     def testFetch(self):
         """ 
@@ -201,8 +235,15 @@ class testImapServer(unittest.TestCase):
         self.o.append('tmpbox','','',str(m))
         self.o.select('tmpbox')
         self.assertEquals(self.o.fetch("1:*","(Flags)")[0],'OK')
-        id=self.o.recent()[1][0]
+        id=1
         
+        # OE query
+        result=self.o.fetch(id,"(BODY.PEEK[HEADER.FIELDS (References X-Ref X-Priority X-MSMail-Priority X-MSOESRec Newsgroups)] ENVELOPE RFC822.SIZE UID FLAGS INTERNALDATE)")
+        self.assertEquals(len(result[1]),5)
+        expect=[(' (("somewhere.foo" NIL "somewher" "foo.org")) (("somewhere.foo" NIL "somewher" "foo.org")) (("somewhere.foo" NIL "somewher" "foo.org")) (("test user" NIL "testuser" "foo.org")) ((NIL NIL "somewher" "foo.org")(NIL NIL "other" "bar.org")) NIL {84}', '"Message from "Test User" <testuser@test.org>    of "Sat,\t14 Dec 2002 09:17:00 CST."'), (' {36}', '<"114.5862946l.21522l.0l"@localhost>'), (') BODY[HEADER.FIELDS (References X-Ref X-Priority X-MSMail-Priority X-MSOESRec Newsgroups)] {2}', '\r\n'), ')']
+        self.assertEquals(result[1][1:],expect)
+        self.assertEquals(result[0],'OK')
+
         # fetch complete message. order and number of headers may differ
         result1 = self.o.fetch(id,"(UID BODY[])")
         result2 = self.o.fetch(id,"(UID RFC822)")
@@ -219,7 +260,7 @@ class testImapServer(unittest.TestCase):
         self.assertEquals(result[0],'OK')
 
         result = self.o.fetch(id,"(ENVELOPE)")
-        expect = [('1 (ENVELOPE ("Mon, 26 Sep 2005 13:26:39 +0200" {21}','dbmail "test" message'), (' (("somewhere.foo" NIL "somewher" "foo.org")) (("somewhere.foo" NIL "somewher" "foo.org")) (("somewhere.foo" NIL "somewher" "foo.org")) (("test user" NIL "testuser" "foo.org")) ((NIL NIL "somewher" "foo.org")(NIL NIL "other" "bar.org")) NIL {82}', 'Message from "Test User" <testuser@test.org>    of "Sat,\t14 Dec 2002 09:17:00 CST.'), (' {36}', '<"114.5862946l.21522l.0l"@localhost>'), '))']
+        expect = [('1 (ENVELOPE ("Mon, 26 Sep 2005 13:26:39 +0200" {21}','dbmail "test" message'), (' (("somewhere.foo" NIL "somewher" "foo.org")) (("somewhere.foo" NIL "somewher" "foo.org")) (("somewhere.foo" NIL "somewher" "foo.org")) (("test user" NIL "testuser" "foo.org")) ((NIL NIL "somewher" "foo.org")(NIL NIL "other" "bar.org")) NIL {84}', '"Message from "Test User" <testuser@test.org>    of "Sat,\t14 Dec 2002 09:17:00 CST."'), (' {36}', '<"114.5862946l.21522l.0l"@localhost>'), '))']
         self.assertEquals(result[0],'OK')
         self.assertEquals(result[1],expect)
 
@@ -232,10 +273,6 @@ class testImapServer(unittest.TestCase):
         self.assertEquals(result[0],'OK')
         self.assertEquals(result[1][0][1][-2:],'\r\n')
         
-        # OE query
-        result=self.o.fetch(id,"(BODY.PEEK[HEADER.FIELDS (References X-Ref X-Priority X-MSMail-Priority X-MSOESRec Newsgroups)] ENVELOPE RFC822.SIZE UID FLAGS INTERNALDATE)")
-        self.assertEquals(result[0],'OK')
-
         # TB query
         result=self.o.fetch(id,"(UID RFC822.SIZE FLAGS BODY.PEEK[HEADER.FIELDS (From To Cc Subject Date Message-ID Priority X-Priority References Newsgroups In-Reply-To Content-Type)])")
         self.assertEquals(result[0],'OK')
@@ -249,7 +286,7 @@ class testImapServer(unittest.TestCase):
         `getacl(mailbox)'
             Get the `ACL's for MAILBOX. 
         """
-        self.assertEquals(self.o.getacl('INBOX'),('OK', ['"INBOX" testuser1 lrswipcda']))
+        self.assertEquals(self.o.getacl('INBOX'),('OK', ['"INBOX" "testuser1" lrswipcda']))
         
     def getQuota(self):
         """ 
@@ -340,8 +377,9 @@ class testImapServer(unittest.TestCase):
         for mailbox in mailboxes:
             self.o.create(mailbox)
             self.o.subscribe(mailbox)
-        self.assertEquals('(\\hasnochildren) "/" "%s"' % mailboxes[6], self.o.lsub()[1][7])
-        self.assertEquals('(\\hasnochildren) "/" "%s"' % mailboxes[2], self.o.lsub('""','"*"')[1][3])
+        result = self.o.lsub()[1]
+        self.assertEquals('(\\hasnochildren) "/" "%s"' % mailboxes[6], result[6])
+        self.assertEquals('(\\hasnochildren) "/" "%s"' % mailboxes[2], self.o.lsub('""','"*"')[1][2])
         self.assertEquals('(\\hasnochildren) "/" "%s"' % mailboxes[2], self.o.lsub('"%s/"' % mailboxes[1],'"%"')[1][0])
 
     def testNoop(self):
@@ -350,6 +388,19 @@ class testImapServer(unittest.TestCase):
              Send `NOOP' to server.
         """
         self.assertEquals(self.o.noop(),('OK', ['NOOP completed']))
+        
+        self.o.select('INBOX')
+        
+        p = getsock()
+        p.debug = DEBUG
+        p.login('testuser1','test'),('OK',['LOGIN completed'])
+        p.select('INBOX')
+
+        print self.o.append('INBOX','\Flagged',"\" 3-Mar-2006 07:15:00 +0200 \"",str(TESTMSG['strict822']))
+
+        result = p.noop()
+        print result
+
 
     def testPartial(self):
         """ 
@@ -374,7 +425,16 @@ class testImapServer(unittest.TestCase):
             Prompt server for an update. Returned data is `None' if no new
             messages, else value of `RECENT' response.
         """
-        self.assertEquals(self.o.recent(),('OK', [None]))
+        readonly=1
+
+        getFreshbox('recenttestbox')
+        self.o.select('recenttestbox',readonly)
+        self.o.fetch("1:*","(Flags)")
+        self.assertEquals(int(self.o.recent()[1][0]) > 0, True)
+
+        self.o.select('recenttestbox')
+        self.o.fetch("1:*","(Flags)")
+        self.assertEquals(self.o.status("recenttestbox",'(RECENT)')[1][0], '"recenttestbox" (RECENT 0)')
 
     def testRename(self):
         """ 
@@ -514,7 +574,29 @@ class testImapServer(unittest.TestCase):
         store(message_set, command, flag_list)
             Alters flag dispositions for messages in mailbox.
         """
-        #self.fail(unimplementedError)
+        self.o.select('INBOX')
+        self.assertEquals(self.o.store('1:*', '+FLAGS', '\Deleted')[0],'OK')
+
+        p = getsock()
+        p.debug = DEBUG
+        p.login('testuser1','test'),('OK',['LOGIN completed'])
+        p.select('INBOX')
+
+        self.assertEquals(self.o.store('1:*', '-FLAGS', '\Deleted')[0],'OK')
+        self.assertRaises(self.o.error,self.o.store, '1:*', '-FLAGS', '\Recent')
+        
+        print p.noop()
+        self.assertEquals(self.o.store('1:*', '+FLAGS', '\Deleted')[0],'OK')
+        print p.noop()
+        print self.o.expunge()
+        print p.noop()
+
+        self.assertEquals(self.o.append('INBOX',(),"",str(TESTMSG['strict822']))[0],'OK')
+        self.assertEquals(self.o.append('INBOX',(),"",str(TESTMSG['strict822']))[0],'OK')
+        self.assertEquals(self.o.append('INBOX',(),"",str(TESTMSG['strict822']))[0],'OK')
+        print p.noop()
+        print p.store('1:*', '+FLAGS', '\Flagged')
+        print self.o.noop()
         
     def testSubscribe(self):
         """
@@ -537,6 +619,10 @@ class testImapServer(unittest.TestCase):
         self.assertEquals(len(result[1]) < 10, True)
         result=self.o.uid('FETCH','10:*', 'FLAGS')
         self.assertEquals(len(result[1]) > 0, True)
+        print self.o.create('testuidcopy')
+        result=self.o.uid('COPY','*','testuidcopy')
+        print result
+        self.assertEquals(result[0],'OK')
         
     def testUnsubscribe(self):
         """
@@ -551,7 +637,7 @@ class testImapServer(unittest.TestCase):
 	    TBD
 	"""
         self.o.select('INBOX')
-        self.assertRaises(self.o.error, self.o.fetch, "1" + " "*12000,"(Flags)")
+        self.assertRaises(self.o.error, self.o.fetch, "1" + " "*120000,"(Flags)")
         
     def tearDown(self):
         try:
@@ -560,7 +646,7 @@ class testImapServer(unittest.TestCase):
                 d=re.match('(\(.*\)) "([^"])" "([^"]+)"',d).groups()[-1]
                 dirs.append(d)
             for i in range(0,len(dirs)):
-                if dirs[-i]=='INBOX': continue
+                if dirs[-i]=='INBOX' or dirs[-i][0] == '#': continue
                 self.o.delete(dirs[-i])
             self.o.logout()
         except: pass
