@@ -18,7 +18,7 @@
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* $Id: user.c 2143 2006-06-02 15:09:15Z aaron $
+/* 
  * This is the dbmail-user program
  * It makes adding users easier 
  *
@@ -35,11 +35,11 @@ char *configFile = DEFAULT_CONFIG_FILE;
 extern db_param_t _db_params;
 
 /* UI policy */
-extern int yes_to_all;
-extern int no_to_all;
-extern int verbose;
-extern int quiet; 		/* Don't be helpful. */
-extern int reallyquiet;	/* Don't print errors. */
+int yes_to_all;
+int no_to_all;
+int verbose;
+int quiet; 		/* Don't be helpful. */
+int reallyquiet;	/* Don't print errors. */
 
 int do_showhelp(void) {
 	
@@ -52,8 +52,7 @@ int do_showhelp(void) {
 	printf("     -e user   empty all mailboxes for a user\n");
 	printf("     -l uspec  list information for matching users\n");
 	printf("     -x alias  create an external forwarding address\n");
-	printf("\n");
-	printf("Summary of options for all modes:\n");
+	printf("\nSummary of options for all modes:\n");
 	printf("     -w passwd specify user's password on the command line\n");
 	printf("     -W [file] read from a file or prompt for a user's password\n");
 	printf("     -p pwtype password type may be one of the following:\n"
@@ -70,11 +69,12 @@ int do_showhelp(void) {
 	printf("     -S alia.. removes a list of recipient aliases (wildcards supported)\n");
 	printf("     -t fwds.. adds a list of deliver-to forwards\n");
 	printf("     -T fwds.. removes a list of deliver-to forwards (wildcards supported)\n");
-	printf("\n");
-        printf("Common options for all DBMail utilities:\n");
+        printf("\nCommon options for all DBMail utilities:\n");
 	printf("     -f file   specify an alternative config file\n");
 	printf("     -q        quietly skip interactive prompts\n"
 	       "               use twice to suppress error messages\n");
+	printf("     -n        show the intended action but do not perform it, no to all\n");
+	printf("     -y        perform all proposed actions, as though yes to all\n");
 	printf("     -v        verbose details\n");
 	printf("     -V        show the version\n");
 	printf("     -h        show this help message\n");
@@ -250,14 +250,12 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'n':
-			printf("-n switch is not supported in this "
-			       "version.\n");
-			return 1;
+			no_to_all = 1;
+			break;
 
 		case 'y':
-			printf("-y switch is not supported in this "
-			       "version.\n");
-			return 1;
+			yes_to_all = 1;
+			break;
 
 		case 'q':
 			/* If we get q twice, be really quiet! */
@@ -274,13 +272,13 @@ int main(int argc, char *argv[])
 
 		case 'V':
 			/* Show the version and return non-zero. */
-			printf("\n*** DBMAIL: dbmail-users version "
-			       "$Revision: 2143 $ %s\n\n", COPYRIGHT);
+			PRINTF_THIS_IS_DBMAIL;
 			result = 1;
 			break;
 
 		default:
-			/* printf("unrecognized option [%c], continuing...\n",optopt); */
+			printf("unrecognized option [%c]\n", optopt); 
+			show_help = 1;
 			break;
 		}
 
@@ -291,7 +289,7 @@ int main(int argc, char *argv[])
 	}	
 
 	/* If nothing is happening, show the help text. */
-	if (!mode || mode_toomany || show_help) {
+	if (!mode || mode_toomany || show_help || (no_to_all && yes_to_all)) {
 		do_showhelp();
 		result = 1;
 		goto freeall;
@@ -367,7 +365,7 @@ int main(int argc, char *argv[])
 			if (pw[len-1] == '\n')
 			        pw[len-1] = '\0';
 			/* fgets guarantees a nul terminated string. */
-			passwd = dm_strdup(pw);
+			passwd = g_strdup(pw);
 
 			/* Restore the previous terminal state (with echo back on). */
 			tcsetattr(fileno(stdin), TCSANOW, &oldattr);
@@ -432,20 +430,16 @@ freeall:
 
 	/* Free the lists. */
 	if (alias_del) {
-		g_list_foreach(alias_del, (GFunc)g_free, NULL);
-		g_list_free(alias_del);
+		g_list_destroy(alias_del);
 	}
 	if (alias_add) {
-		g_list_foreach(alias_add, (GFunc)g_free, NULL);
-		g_list_free(alias_add);
+		g_list_destroy(alias_add);
 	}
 	if (fwds_del) {
-		g_list_foreach(fwds_del, (GFunc)g_free, NULL);
-		g_list_free(fwds_del);
+		g_list_destroy(fwds_del);
 	}
 	if (fwds_add) {
-		g_list_foreach(fwds_add, (GFunc)g_free, NULL);
-		g_list_free(fwds_add);
+		g_list_destroy(fwds_add);
 	}
 
 	db_disconnect();
